import os
import numpy as np

from wisdem.aeroelasticse.CaseGen_General import CaseGen_General
from wisdem.aeroelasticse.CaseGen_IEC import CaseGen_IEC

# def power_curve_fit(fst_vt, runDir, namebase, TMax, turbine_class, turbulence_class, Vrated, U_init=[], Omega_init=[], pitch_init=[], Turbsim_exe='', ptfm_U_init=[], ptfm_pitch_init=[], ptfm_surge_init=[], ptfm_heave_init=[], metocean_U_init=[], metocean_Hs_init=[], metocean_Tp_init=[]):

#     # Default Runtime
#     T      = 240.
#     TStart = 120.
#     # T      = 120.
#     # TStart = 60.
    
#     # Overwrite for testing
#     if TMax < T:
#         T      = TMax
#         TStart = 0.

#     # Run conditions for points which will be used for a cubic polynomial fit
#     # U = [10.]
#     U = [4.,8.,9.,10.]
#     omega = np.interp(U, U_init, Omega_init)
#     pitch = np.interp(U, U_init, pitch_init)

#     # Check if floating
#     floating_dof = [fst_vt['ElastoDyn']['PtfmSgDOF'], fst_vt['ElastoDyn']['PtfmSwDOF'], fst_vt['ElastoDyn']['PtfmHvDOF'], fst_vt['ElastoDyn']['PtfmRDOF'], fst_vt['ElastoDyn']['PtfmPDOF'], fst_vt['ElastoDyn']['PtfmYDOF']]
#     if any(floating_dof):
#         floating = True
#         if ptfm_U_init == []:
#             ptfm_U_init     = [4., 5., 6., 7., 8., 9., 10., 10.5, 11., 12., 14., 19., 24.]
#             ptfm_surge_init = [3.8758245863838807, 5.57895688031965, 7.619719770801395, 9.974666446553552, 12.675469235464321, 16.173740623041965, 20.069526574594757, 22.141906121375552, 23.835466098954708, 22.976075549477354, 17.742743260748373, 14.464576583154068, 14.430969814391759]
#             ptfm_heave_init = [0.030777174904620515, 0.008329930604820483, -0.022973502300090893, -0.06506947653943342, -0.12101317451310406, -0.20589689839069836, -0.3169518280533253, -0.3831692055885472, -0.4409624802614755, -0.41411738171337675, -0.2375323506471747, -0.1156867221814119, -0.07029955933167854]
#             ptfm_pitch_init = [0.7519976895165884, 1.104483050851386, 1.5180416334025146, 1.9864587671004394, 2.5152769741130134, 3.1937704945765795, 3.951314212429935, 4.357929703098016, 4.693765745171944, 4.568760630312074, 3.495057478277534, 2.779958240049992, 2.69008798174216]
#         if metocean_U_init == []:
#             metocean_U_init  = [4.00, 6.00, 8.00, 10.00, 12.00, 14.00, 16.00, 18.00, 20.00, 22.00, 24.00]
#             metocean_Hs_init = [1.908567568, 1.960162595, 2.062722244, 2.224539415, 2.489931091, 2.802984019, 3.182301485, 3.652236101, 4.182596165, 4.695439504, 5.422289377]
#             metocean_Tp_init = [12.23645701, 12.14497777, 11.90254947, 11.5196666, 11.05403739, 10.65483551, 10.27562225, 10.13693777, 10.27842325, 10.11660396, 10.96177917]

#         ptfm_heave = np.interp(U, ptfm_U_init, ptfm_heave_init)
#         ptfm_surge = np.interp(U, ptfm_U_init, ptfm_surge_init)
#         ptfm_pitch = np.interp(U, ptfm_U_init, ptfm_pitch_init)
#         metocean_Hs = np.interp(U, metocean_U_init, metocean_Hs_init)
#         metocean_Tp = np.interp(U, metocean_U_init, metocean_Tp_init)
#     else:
#         floating = False

#     case_inputs = {}
#     # simulation settings
#     # case_inputs[("ElastoDyn","PtfmSgDOF")]     = {'vals':['False'], 'group':0}
#     # case_inputs[("ElastoDyn","PtfmHvDOF")]     = {'vals':['False'], 'group':0}
#     # case_inputs[("ElastoDyn","PtfmPDOF")]     = {'vals':['False'], 'group':0}
#     case_inputs[("ElastoDyn","PtfmSwDOF")]     = {'vals':['False'], 'group':0}
#     case_inputs[("ElastoDyn","PtfmRDOF")]     = {'vals':['False'], 'group':0}
#     case_inputs[("ElastoDyn","PtfmYDOF")]     = {'vals':['False'], 'group':0}

#     case_inputs[("Fst","TMax")] = {'vals':[T], 'group':0}
#     case_inputs[("Fst","TStart")] = {'vals':[TStart], 'group':0}
#     case_inputs[("ElastoDyn","YawDOF")]      = {'vals':['True'], 'group':0}
#     case_inputs[("ElastoDyn","FlapDOF1")]    = {'vals':['True'], 'group':0}
#     case_inputs[("ElastoDyn","FlapDOF2")]    = {'vals':['True'], 'group':0}
#     case_inputs[("ElastoDyn","EdgeDOF")]     = {'vals':['True'], 'group':0}
#     case_inputs[("ElastoDyn","DrTrDOF")]     = {'vals':['False'], 'group':0}
#     case_inputs[("ElastoDyn","GenDOF")]      = {'vals':['True'], 'group':0} 
#     case_inputs[("ElastoDyn","TwFADOF1")]    = {'vals':['False'], 'group':0}
#     case_inputs[("ElastoDyn","TwFADOF2")]    = {'vals':['False'], 'group':0}
#     case_inputs[("ElastoDyn","TwSSDOF1")]    = {'vals':['False'], 'group':0}
#     case_inputs[("ElastoDyn","TwSSDOF2")]    = {'vals':['False'], 'group':0}
#     case_inputs[("ServoDyn","PCMode")]       = {'vals':[5], 'group':0}
#     case_inputs[("ServoDyn","VSContrl")]     = {'vals':[5], 'group':0}
#     case_inputs[("ServoDyn","YCMode")]       = {'vals':[5], 'group':0}
#     case_inputs[("AeroDyn15","WakeMod")]     = {'vals':[1], 'group':0}
#     case_inputs[("AeroDyn15","AFAeroMod")]   = {'vals':[2], 'group':0}
#     case_inputs[("AeroDyn15","TwrPotent")]   = {'vals':[0], 'group':0}
#     case_inputs[("AeroDyn15","TwrShadow")]   = {'vals':['False'], 'group':0}
#     case_inputs[("AeroDyn15","TwrAero")]     = {'vals':['False'], 'group':0}
#     case_inputs[("AeroDyn15","SkewMod")]     = {'vals':[1], 'group':0}
#     case_inputs[("AeroDyn15","TipLoss")]     = {'vals':['True'], 'group':0}
#     case_inputs[("AeroDyn15","HubLoss")]     = {'vals':['True'], 'group':0}
#     case_inputs[("AeroDyn15","TanInd")]      = {'vals':['True'], 'group':0}
#     case_inputs[("AeroDyn15","AIDrag")]      = {'vals':['True'], 'group':0}
#     case_inputs[("AeroDyn15","TIDrag")]      = {'vals':['True'], 'group':0}
#     case_inputs[("AeroDyn15","IndToler")]    = {'vals':[1.e-5], 'group':0}
#     case_inputs[("AeroDyn15","MaxIter")]     = {'vals':[5000], 'group':0}
#     case_inputs[("AeroDyn15","UseBlCm")]     = {'vals':['True'], 'group':0}
#     # inital conditions
#     case_inputs[("InflowWind","WindType")] = {'vals':[1], 'group':0}
#     case_inputs[("InflowWind","HWindSpeed")] = {'vals':U, 'group':1}
#     case_inputs[("ElastoDyn","RotSpeed")] = {'vals':omega, 'group':1}
#     case_inputs[("ElastoDyn","BlPitch1")] = {'vals':pitch, 'group':1}
#     case_inputs[("ElastoDyn","BlPitch2")] = case_inputs[("ElastoDyn","BlPitch1")]
#     case_inputs[("ElastoDyn","BlPitch3")] = case_inputs[("ElastoDyn","BlPitch1")]
#     if floating == True:
#         case_inputs[("ElastoDyn","PtfmSurge")] = {'vals':ptfm_surge, 'group':1}
#         case_inputs[("ElastoDyn","PtfmHeave")] = {'vals':ptfm_heave, 'group':1}
#         case_inputs[("ElastoDyn","PtfmPitch")] = {'vals':ptfm_pitch, 'group':1}
#         case_inputs[("HydroDyn","WaveHs")] = {'vals':metocean_Hs, 'group':1}
#         case_inputs[("HydroDyn","WaveTp")] = {'vals':metocean_Tp, 'group':1}
#         case_inputs[("HydroDyn","RdtnDT")] = {'vals':[fst_vt["Fst"]["DT"]], 'group':0}
#         case_inputs[("HydroDyn","WaveMod")] = {'vals':[1], 'group':0}

#     from CaseGen_General import CaseGen_General
#     case_list, case_name_list = CaseGen_General(case_inputs, dir_matrix=runDir, namebase=namebase)

#     channels = ['Wind1VelX','GenPwr']

#     return case_list, case_name_list, channels

def power_curve(fst_vt, runDir, namebase, TMax, turbine_class, turbulence_class, Vrated, U_init=[], Omega_init=[], pitch_init=[], Turbsim_exe='', ptfm_U_init=[], ptfm_pitch_init=[], ptfm_surge_init=[], ptfm_heave_init=[], metocean_U_init=[], metocean_Hs_init=[], metocean_Tp_init=[], V_R25=0.):

    # Default Runtime
    T      = 360.
    TStart = 120.
    # T      = 120.
    # TStart = 60.
    
    # Overwrite for testing
    if TMax < T:
        T      = TMax
        TStart = 0.

    # Run conditions
    U_all = list(sorted([4., 6., 8., 9., 10., 10.5, 11., 11.5, 11.75, 12., 12.5, 13., 14., 19., 25., Vrated]))
    if V_R25 != 0.:
        U_all.append(V_R25)
        U_all = list(sorted(U_all))
    U = [Vi for Vi in U_all if Vi <= Vrated]
    # print(U)

    # dt = [0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001]
    dt = [0.01]*len(U)

    # U = [4.,8.,9.,10.]
    omega = np.interp(U, U_init, Omega_init)
    pitch = np.interp(U, U_init, pitch_init)
    for i, (omegai, pitchi) in enumerate(zip(omega, pitch)):
        if pitchi > 0. and omegai < Omega_init[-1]:
            pitch[i] = 0.

    # Check if floating
    floating_dof = [fst_vt['ElastoDyn']['PtfmSgDOF'], fst_vt['ElastoDyn']['PtfmSwDOF'], fst_vt['ElastoDyn']['PtfmHvDOF'], fst_vt['ElastoDyn']['PtfmRDOF'], fst_vt['ElastoDyn']['PtfmPDOF'], fst_vt['ElastoDyn']['PtfmYDOF']]
    if any(floating_dof):
        floating = True
        if ptfm_U_init == []:
            ptfm_U_init     = [3., 5., 6., 7., 8., 9., 10., 10.5, 11., 12., 14., 19., 25.]
            ptfm_surge_init = [3.8758245863838807, 5.57895688031965, 7.619719770801395, 9.974666446553552, 12.675469235464321, 16.173740623041965, 20.069526574594757, 22.141906121375552, 23.835466098954708, 22.976075549477354, 17.742743260748373, 14.464576583154068, 14.430969814391759]
            ptfm_heave_init = [0.030777174904620515, 0.008329930604820483, -0.022973502300090893, -0.06506947653943342, -0.12101317451310406, -0.20589689839069836, -0.3169518280533253, -0.3831692055885472, -0.4409624802614755, -0.41411738171337675, -0.2375323506471747, -0.1156867221814119, -0.07029955933167854]
            ptfm_pitch_init = [0.7519976895165884, 1.104483050851386, 1.5180416334025146, 1.9864587671004394, 2.5152769741130134, 3.1937704945765795, 3.951314212429935, 4.357929703098016, 4.693765745171944, 4.568760630312074, 3.495057478277534, 2.779958240049992, 2.69008798174216]
        if metocean_U_init == []:
            metocean_U_init  = [3.00, 6.00, 8.00, 10.00, 12.00, 14.00, 16.00, 18.00, 20.00, 22.00, 25.00]
            metocean_Hs_init = [1.908567568, 1.960162595, 2.062722244, 2.224539415, 2.489931091, 2.802984019, 3.182301485, 3.652236101, 4.182596165, 4.695439504, 5.422289377]
            metocean_Tp_init = [12.23645701, 12.14497777, 11.90254947, 11.5196666, 11.05403739, 10.65483551, 10.27562225, 10.13693777, 10.27842325, 10.11660396, 10.96177917]

        ptfm_heave = np.interp(U, ptfm_U_init, ptfm_heave_init)
        ptfm_surge = np.interp(U, ptfm_U_init, ptfm_surge_init)
        ptfm_pitch = np.interp(U, ptfm_U_init, ptfm_pitch_init)
        metocean_Hs = np.interp(U, metocean_U_init, metocean_Hs_init)
        metocean_Tp = np.interp(U, metocean_U_init, metocean_Tp_init)
    else:
        floating = False

    case_inputs = {}
    # simulation settings
    # case_inputs[("ElastoDyn","PtfmSgDOF")]     = {'vals':['False'], 'group':0}
    # case_inputs[("ElastoDyn","PtfmHvDOF")]     = {'vals':['False'], 'group':0}
    # case_inputs[("ElastoDyn","PtfmPDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","PtfmSwDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","PtfmRDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","PtfmYDOF")]     = {'vals':['False'], 'group':0}

    case_inputs[("Fst","TMax")] = {'vals':[T], 'group':0}
    case_inputs[("Fst","TStart")] = {'vals':[TStart], 'group':0}
    case_inputs[("Fst","DT")] = {'vals':dt, 'group':1}
    case_inputs[("ElastoDyn","YawDOF")]      = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF2")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","EdgeDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","DrTrDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","GenDOF")]      = {'vals':['True'], 'group':0} 
    case_inputs[("ElastoDyn","TwFADOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwFADOF2")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF2")]    = {'vals':['False'], 'group':0}
    case_inputs[("ServoDyn","PCMode")]       = {'vals':[5], 'group':0}
    case_inputs[("ServoDyn","VSContrl")]     = {'vals':[5], 'group':0}
    case_inputs[("ServoDyn","YCMode")]       = {'vals':[5], 'group':0}
    case_inputs[("AeroDyn15","WakeMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","AFAeroMod")]   = {'vals':[2], 'group':0}
    case_inputs[("AeroDyn15","TwrPotent")]   = {'vals':[0], 'group':0}
    case_inputs[("AeroDyn15","TwrShadow")]   = {'vals':['False'], 'group':0}
    case_inputs[("AeroDyn15","TwrAero")]     = {'vals':['False'], 'group':0}
    case_inputs[("AeroDyn15","SkewMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","TipLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","HubLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TanInd")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","AIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","IndToler")]    = {'vals':[1.e-5], 'group':0}
    case_inputs[("AeroDyn15","MaxIter")]     = {'vals':[5000], 'group':0}
    case_inputs[("AeroDyn15","UseBlCm")]     = {'vals':['True'], 'group':0}
    # inital conditions
    case_inputs[("InflowWind","WindType")] = {'vals':[1], 'group':0}
    case_inputs[("InflowWind","HWindSpeed")] = {'vals':U, 'group':1}
    case_inputs[("ElastoDyn","RotSpeed")] = {'vals':omega, 'group':1}
    case_inputs[("ElastoDyn","BlPitch1")] = {'vals':pitch, 'group':1}
    case_inputs[("ElastoDyn","BlPitch2")] = case_inputs[("ElastoDyn","BlPitch1")]
    case_inputs[("ElastoDyn","BlPitch3")] = case_inputs[("ElastoDyn","BlPitch1")]
    if floating == True:
        case_inputs[("ElastoDyn","PtfmSurge")] = {'vals':ptfm_surge, 'group':1}
        case_inputs[("ElastoDyn","PtfmHeave")] = {'vals':ptfm_heave, 'group':1}
        case_inputs[("ElastoDyn","PtfmPitch")] = {'vals':ptfm_pitch, 'group':1}
        case_inputs[("HydroDyn","WaveHs")] = {'vals':metocean_Hs, 'group':1}
        case_inputs[("HydroDyn","WaveTp")] = {'vals':metocean_Tp, 'group':1}
        case_inputs[("HydroDyn","RdtnDT")] = {'vals':dt, 'group':1}
        case_inputs[("HydroDyn","WaveMod")] = {'vals':[1], 'group':0}

    from wisdem.aeroelasticse.CaseGen_General import CaseGen_General
    case_list, case_name_list = CaseGen_General(case_inputs, dir_matrix=runDir, namebase=namebase)

    channels = ['Wind1VelX','GenPwr',"RtAeroCp", "RotTorq", "RotThrust", "RotSpeed", "BldPitch1"]

    return case_list, case_name_list, channels

def RotorSE_rated(fst_vt, runDir, namebase, TMax, turbine_class, turbulence_class, Vrated, U_init=[], Omega_init=[], pitch_init=[], Turbsim_exe='', ptfm_U_init=[], ptfm_pitch_init=[], ptfm_surge_init=[], ptfm_heave_init=[], metocean_U_init=[], metocean_Hs_init=[], metocean_Tp_init=[]):

    # Default Runtime
    T      = 240.
    TStart = 120.

    # dt = 0.001
    dt = 0.01
    
    # Overwrite for testing
    if TMax < T:
        T      = TMax
        TStart = 0.

    omega = np.interp(Vrated, U_init, Omega_init)
    pitch = np.interp(Vrated, U_init, pitch_init)

    # Check if floating
    floating_dof = [fst_vt['ElastoDyn']['PtfmSgDOF'], fst_vt['ElastoDyn']['PtfmSwDOF'], fst_vt['ElastoDyn']['PtfmHvDOF'], fst_vt['ElastoDyn']['PtfmRDOF'], fst_vt['ElastoDyn']['PtfmPDOF'], fst_vt['ElastoDyn']['PtfmYDOF']]
    if any(floating_dof):
        floating = True
        if ptfm_U_init == []:
            ptfm_U_init     = [4., 5., 6., 7., 8., 9., 10., 10.5, 11., 12., 14., 19., 24.]
            ptfm_surge_init = [3.8758245863838807, 5.57895688031965, 7.619719770801395, 9.974666446553552, 12.675469235464321, 16.173740623041965, 20.069526574594757, 22.141906121375552, 23.835466098954708, 22.976075549477354, 17.742743260748373, 14.464576583154068, 14.430969814391759]
            ptfm_heave_init = [0.030777174904620515, 0.008329930604820483, -0.022973502300090893, -0.06506947653943342, -0.12101317451310406, -0.20589689839069836, -0.3169518280533253, -0.3831692055885472, -0.4409624802614755, -0.41411738171337675, -0.2375323506471747, -0.1156867221814119, -0.07029955933167854]
            ptfm_pitch_init = [0.7519976895165884, 1.104483050851386, 1.5180416334025146, 1.9864587671004394, 2.5152769741130134, 3.1937704945765795, 3.951314212429935, 4.357929703098016, 4.693765745171944, 4.568760630312074, 3.495057478277534, 2.779958240049992, 2.69008798174216]
        if metocean_U_init == []:
            metocean_U_init  = [4.00, 6.00, 8.00, 10.00, 12.00, 14.00, 16.00, 18.00, 20.00, 22.00, 24.00]
            metocean_Hs_init = [1.908567568, 1.960162595, 2.062722244, 2.224539415, 2.489931091, 2.802984019, 3.182301485, 3.652236101, 4.182596165, 4.695439504, 5.422289377]
            metocean_Tp_init = [12.23645701, 12.14497777, 11.90254947, 11.5196666, 11.05403739, 10.65483551, 10.27562225, 10.13693777, 10.27842325, 10.11660396, 10.96177917]

        ptfm_heave = [np.interp(Vrated, ptfm_U_init, ptfm_heave_init)]
        ptfm_surge = [np.interp(Vrated, ptfm_U_init, ptfm_surge_init)]
        ptfm_pitch = [np.interp(Vrated, ptfm_U_init, ptfm_pitch_init)]
        metocean_Hs = [np.interp(Vrated, metocean_U_init, metocean_Hs_init)]
        metocean_Tp = [np.interp(Vrated, metocean_U_init, metocean_Tp_init)]
    else:
        floating = False

    case_inputs = {}
    case_inputs[("Fst","TMax")]              = {'vals':[T], 'group':0}
    case_inputs[("Fst","TStart")]            = {'vals':[TStart], 'group':0}
    case_inputs[("Fst","DT")]                = {'vals':[dt], 'group':0}
    case_inputs[("Fst","OutFileFmt")]        = {'vals':[2], 'group':0}
    
    case_inputs[("InflowWind","WindType")]   = {'vals':[1], 'group':0}
    case_inputs[("InflowWind","HWindSpeed")] = {'vals':[Vrated], 'group':0}

    case_inputs[("ElastoDyn","RotSpeed")]    = {'vals':[omega], 'group':0}
    case_inputs[("ElastoDyn","BlPitch1")]    = {'vals':[pitch], 'group':0}
    case_inputs[("ElastoDyn","BlPitch2")]    = {'vals':[pitch], 'group':0}
    case_inputs[("ElastoDyn","BlPitch3")]    = {'vals':[pitch], 'group':0}
    case_inputs[("ElastoDyn","YawDOF")]      = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF2")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","EdgeDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","DrTrDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","GenDOF")]      = {'vals':['True'], 'group':0} 
    case_inputs[("ElastoDyn","TwFADOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwFADOF2")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF2")]    = {'vals':['False'], 'group':0}

    case_inputs[("ServoDyn","PCMode")]       = {'vals':[5], 'group':0}
    case_inputs[("ServoDyn","VSContrl")]     = {'vals':[5], 'group':0}

    case_inputs[("AeroDyn15","WakeMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","AFAeroMod")]   = {'vals':[2], 'group':0}
    case_inputs[("AeroDyn15","TwrPotent")]   = {'vals':[0], 'group':0}
    case_inputs[("AeroDyn15","TwrShadow")]   = {'vals':['False'], 'group':0}
    case_inputs[("AeroDyn15","TwrAero")]     = {'vals':['False'], 'group':0}
    case_inputs[("AeroDyn15","SkewMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","TipLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","HubLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TanInd")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","AIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","IndToler")]    = {'vals':[1.e-5], 'group':0}
    case_inputs[("AeroDyn15","MaxIter")]     = {'vals':[5000], 'group':0}
    case_inputs[("AeroDyn15","UseBlCm")]     = {'vals':['True'], 'group':0}
    
    if floating == True:
        case_inputs[("ElastoDyn","PtfmSurge")] = {'vals':ptfm_surge, 'group':1}
        case_inputs[("ElastoDyn","PtfmHeave")] = {'vals':ptfm_heave, 'group':1}
        case_inputs[("ElastoDyn","PtfmPitch")] = {'vals':ptfm_pitch, 'group':1}
        case_inputs[("HydroDyn","WaveHs")] = {'vals':metocean_Hs, 'group':1}
        case_inputs[("HydroDyn","WaveTp")] = {'vals':metocean_Tp, 'group':1}
        case_inputs[("HydroDyn","RdtnDT")] = {'vals':[dt], 'group':0}
        case_inputs[("HydroDyn","WaveMod")] = {'vals':[1], 'group':0}

    namebase += '_rated'
    case_list, case_name_list = CaseGen_General(case_inputs, dir_matrix=runDir, namebase=namebase)

    channels  = ["TipDxc1", "TipDyc1"]
    channels += ["RootMxc1", "RootMyc1", "RootMzc1", "RootMxc2", "RootMyc2", "RootMzc2", "RootMxc3", "RootMyc3", "RootMzc3"]
    channels += ["RootFxc1", "RootFyc1", "RootFzc1", "RootFxc2", "RootFyc2", "RootFzc2", "RootFxc3", "RootFyc3", "RootFzc3"]
    channels += ["RtAeroCp", "RotTorq", "RotThrust", "RotSpeed"]

    return case_list, case_name_list, channels

def RotorSE_DLC_1_4_Rated(fst_vt, runDir, namebase, TMax, turbine_class, turbulence_class, Vrated, U_init=[], Omega_init=[], pitch_init=[], Turbsim_exe=''):

    # Default Runtime
    T      = 60.
    TStart = 30.
    # TStart = 0.
    
    # Overwrite for testing
    if TMax < T:
        T      = TMax
        TStart = 0.


    iec = CaseGen_IEC()
    iec.init_cond[("ElastoDyn","RotSpeed")] = {'U':  U_init}
    iec.init_cond[("ElastoDyn","RotSpeed")]['val'] = Omega_init
    iec.init_cond[("ElastoDyn","BlPitch1")] = {'U':  U_init}
    iec.init_cond[("ElastoDyn","BlPitch1")]['val'] = pitch_init
    iec.init_cond[("ElastoDyn","BlPitch2")] = iec.init_cond[("ElastoDyn","BlPitch1")]
    iec.init_cond[("ElastoDyn","BlPitch3")] = iec.init_cond[("ElastoDyn","BlPitch1")]
    iec.Turbine_Class = turbine_class
    iec.Turbulence_Class = turbulence_class
    iec.D = fst_vt['ElastoDyn']['TipRad']*2.
    iec.z_hub = fst_vt['InflowWind']['RefHt']

    iec.dlc_inputs = {}
    iec.dlc_inputs['DLC']   = [1.4]
    iec.dlc_inputs['U']     = [[Vrated]]
    iec.dlc_inputs['Seeds'] = [[]]
    iec.dlc_inputs['Yaw']   = [[]]
    iec.transient_dir_change        = '-'  # '+','-','both': sign for transient events in EDC, EWS
    iec.transient_shear_orientation = 'v'  # 'v','h','both': vertical or horizontal shear for EWS

    iec.wind_dir        = runDir
    iec.case_name_base  = namebase + '_gust'
    iec.Turbsim_exe     = ''
    iec.debug_level     = 0
    iec.parallel_windfile_gen = False
    iec.run_dir         = runDir

    case_inputs = {}
    case_inputs[("Fst","TMax")]              = {'vals':[T], 'group':0}
    case_inputs[("Fst","TStart")]            = {'vals':[TStart], 'group':0}
    case_inputs[("Fst","OutFileFmt")]        = {'vals':[2], 'group':0}

    case_inputs[("ElastoDyn","YawDOF")]      = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF1")]    = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF2")]    = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","EdgeDOF")]     = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","DrTrDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","GenDOF")]      = {'vals':['True'], 'group':0} 
    case_inputs[("ElastoDyn","TwFADOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwFADOF2")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF2")]    = {'vals':['False'], 'group':0}

    case_inputs[("ServoDyn","PCMode")]       = {'vals':[5], 'group':0}
    case_inputs[("ServoDyn","VSContrl")]     = {'vals':[5], 'group':0}
    case_inputs[("ServoDyn","YCMode")]       = {'vals':[5], 'group':0}

    case_inputs[("AeroDyn15","WakeMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","AFAeroMod")]   = {'vals':[2], 'group':0}
    case_inputs[("AeroDyn15","TwrPotent")]   = {'vals':[0], 'group':0}
    case_inputs[("AeroDyn15","TwrShadow")]   = {'vals':['False'], 'group':0}
    case_inputs[("AeroDyn15","TwrAero")]     = {'vals':['False'], 'group':0}

    case_inputs[("AeroDyn15","SkewMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","TipLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","HubLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TanInd")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","AIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","IndToler")]    = {'vals':[1.e-5], 'group':0}
    case_inputs[("AeroDyn15","MaxIter")]     = {'vals':[5000], 'group':0}
    case_inputs[("AeroDyn15","UseBlCm")]     = {'vals':['True'], 'group':0}
    
    case_list, case_name_list = iec.execute(case_inputs=case_inputs)

    channels  = ["TipDxc1", "TipDyc1", "TipDzc1", "TipDxc2", "TipDyc2", "TipDzc2", "TipDxc3", "TipDyc3", "TipDzc3"]
    channels += ["RootMxc1", "RootMyc1", "RootMzc1", "RootMxc2", "RootMyc2", "RootMzc2", "RootMxc3", "RootMyc3", "RootMzc3"]
    channels += ["RootFxc1", "RootFyc1", "RootFzc1", "RootFxc2", "RootFyc2", "RootFzc2", "RootFxc3", "RootFyc3", "RootFzc3"]
    channels += ["RtAeroCp", "RotTorq", "RotThrust", "RotSpeed", "NacYaw"]

    channels += ["B1N1Fx", "B1N2Fx", "B1N3Fx", "B1N4Fx", "B1N5Fx", "B1N6Fx", "B1N7Fx", "B1N8Fx", "B1N9Fx"]
    channels += ["B1N1Fy", "B1N2Fy", "B1N3Fy", "B1N4Fy", "B1N5Fy", "B1N6Fy", "B1N7Fy", "B1N8Fy", "B1N9Fy"]

    return case_list, case_name_list, channels

def RotorSE_DLC_7_1_Steady(fst_vt, runDir, namebase, TMax, turbine_class, turbulence_class, U, U_init=[], Omega_init=[], pitch_init=[], Turbsim_exe=''):
    # Extreme 1yr return period wind speed with a power fault resulting in the blade not feathering

    # Default Runtime
    T      = 60.
    TStart = 30.
    
    # Overwrite for testing
    if TMax < T:
        T      = TMax
        TStart = 0.

    Pitch = 0.
    Omega = 0.

    case_inputs = {}
    case_inputs[("Fst","TMax")]              = {'vals':[T], 'group':0}
    case_inputs[("Fst","TStart")]            = {'vals':[TStart], 'group':0}
    case_inputs[("Fst","OutFileFmt")]        = {'vals':[2], 'group':0}
    
    case_inputs[("InflowWind","WindType")]   = {'vals':[1], 'group':0}
    case_inputs[("InflowWind","HWindSpeed")] = {'vals':[U], 'group':0}
    case_inputs[("InflowWind","PLexp")] = {'vals':[0.11], 'group':0}

    case_inputs[("ElastoDyn","RotSpeed")]    = {'vals':[Omega], 'group':0}
    case_inputs[("ElastoDyn","BlPitch1")]    = {'vals':[Pitch], 'group':0}
    case_inputs[("ElastoDyn","BlPitch2")]    = {'vals':[Pitch], 'group':0}
    case_inputs[("ElastoDyn","BlPitch3")]    = {'vals':[Pitch], 'group':0}
    case_inputs[("ElastoDyn","YawDOF")]      = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF1")]    = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF2")]    = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","EdgeDOF")]     = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","DrTrDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","GenDOF")]      = {'vals':['False'], 'group':0} 
    case_inputs[("ElastoDyn","TwFADOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwFADOF2")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF2")]    = {'vals':['False'], 'group':0}

    case_inputs[("ServoDyn","PCMode")]       = {'vals':[0], 'group':0}
    case_inputs[("ServoDyn","VSContrl")]     = {'vals':[5], 'group':0}
    case_inputs[("ServoDyn","YCMode")]       = {'vals':[5], 'group':0}

    case_inputs[("AeroDyn15","WakeMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","AFAeroMod")]   = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","TwrPotent")]   = {'vals':[0], 'group':0}
    case_inputs[("AeroDyn15","TwrShadow")]   = {'vals':['False'], 'group':0}
    case_inputs[("AeroDyn15","TwrAero")]     = {'vals':['False'], 'group':0}

    case_inputs[("AeroDyn15","SkewMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","TipLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","HubLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TanInd")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","AIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","IndToler")]    = {'vals':[1.e-5], 'group':0}
    case_inputs[("AeroDyn15","MaxIter")]     = {'vals':[5000], 'group':0}
    case_inputs[("AeroDyn15","UseBlCm")]     = {'vals':['True'], 'group':0}
    

    namebase += '_idle50yr'
    case_list, case_name_list = CaseGen_General(case_inputs, namebase=namebase, save_matrix=False)

    channels  = ["TipDxc1", "TipDyc1", "TipDzc1", "TipDxc2", "TipDyc2", "TipDzc2", "TipDxc3", "TipDyc3", "TipDzc3"]
    channels += ["RootMxc1", "RootMyc1", "RootMzc1", "RootMxc2", "RootMyc2", "RootMzc2", "RootMxc3", "RootMyc3", "RootMzc3"]
    channels += ["RootFxc1", "RootFyc1", "RootFzc1", "RootFxc2", "RootFyc2", "RootFzc2", "RootFxc3", "RootFyc3", "RootFzc3"]
    channels += ["RtAeroCp", "RotTorq", "RotThrust", "RotSpeed", "NacYaw"]

    channels += ["B1N1Fx", "B1N2Fx", "B1N3Fx", "B1N4Fx", "B1N5Fx", "B1N6Fx", "B1N7Fx", "B1N8Fx", "B1N9Fx"]
    channels += ["B1N1Fy", "B1N2Fy", "B1N3Fy", "B1N4Fy", "B1N5Fy", "B1N6Fy", "B1N7Fy", "B1N8Fy", "B1N9Fy"]

    return case_list, case_name_list, channels


def RotorSE_DLC_1_1_Turb(fst_vt, runDir, namebase, TMax, turbine_class, turbulence_class, U, U_init=[], Omega_init=[], pitch_init=[], Turbsim_exe='', debug_level=0, cores=0, mpi_run=False, mpi_comm_map_down=[]):
    
    # Default Runtime
    T      = 630.
    TStart = 30.
    
    # Overwrite for testing
    if TMax < T:
        T      = TMax
        TStart = 0.


    iec = CaseGen_IEC()
    iec.init_cond[("ElastoDyn","RotSpeed")] = {'U':  U_init}
    iec.init_cond[("ElastoDyn","RotSpeed")]['val'] = [0.95*omega_i for omega_i in Omega_init]
    iec.init_cond[("ElastoDyn","BlPitch1")] = {'U':  U_init}
    iec.init_cond[("ElastoDyn","BlPitch1")]['val'] = pitch_init
    iec.init_cond[("ElastoDyn","BlPitch2")] = iec.init_cond[("ElastoDyn","BlPitch1")]
    iec.init_cond[("ElastoDyn","BlPitch3")] = iec.init_cond[("ElastoDyn","BlPitch1")]
    iec.Turbine_Class = turbine_class
    iec.Turbulence_Class = turbulence_class
    iec.D = fst_vt['ElastoDyn']['TipRad']*2.
    iec.z_hub = fst_vt['InflowWind']['RefHt']

    iec.dlc_inputs = {}
    iec.dlc_inputs['DLC']   = [1.1]
    iec.dlc_inputs['U']     = [[U]]
    # iec.dlc_inputs['Seeds'] = [[1]]
    iec.dlc_inputs['Seeds'] = [[310414237, 1764051066, 1935526301, 333954657, -960771537, 714191176]] # nothing special about these seeds, randomly generated
    iec.dlc_inputs['Yaw']   = [[]]
    iec.transient_dir_change        = '-'  # '+','-','both': sign for transient events in EDC, EWS
    iec.transient_shear_orientation = 'v'  # 'v','h','both': vertical or horizontal shear for EWS

    iec.wind_dir        = runDir
    iec.case_name_base  = namebase + '_turb'
    iec.Turbsim_exe     = Turbsim_exe
    iec.debug_level     = debug_level
    iec.cores           = cores
    iec.run_dir         = runDir
    iec.overwrite       = True
    # iec.overwrite       = False
    if cores > 1:
        iec.parallel_windfile_gen = True
    else:
        iec.parallel_windfile_gen = False

    # mpi_run = False
    if mpi_run:
        iec.mpi_run           = mpi_run
        iec.comm_map_down = mpi_comm_map_down

    case_inputs = {}
    case_inputs[("Fst","TMax")]              = {'vals':[T], 'group':0}
    case_inputs[("Fst","TStart")]            = {'vals':[TStart], 'group':0}
    case_inputs[("Fst","OutFileFmt")]        = {'vals':[2], 'group':0}

    case_inputs[("ElastoDyn","YawDOF")]      = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF1")]    = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","FlapDOF2")]    = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","EdgeDOF")]     = {'vals':['True'], 'group':0}
    case_inputs[("ElastoDyn","DrTrDOF")]     = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","GenDOF")]      = {'vals':['True'], 'group':0} 
    case_inputs[("ElastoDyn","TwFADOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwFADOF2")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF1")]    = {'vals':['False'], 'group':0}
    case_inputs[("ElastoDyn","TwSSDOF2")]    = {'vals':['False'], 'group':0}

    case_inputs[("ServoDyn","PCMode")]       = {'vals':[5], 'group':0}
    case_inputs[("ServoDyn","VSContrl")]     = {'vals':[5], 'group':0}
    case_inputs[("ServoDyn","YCMode")]       = {'vals':[5], 'group':0}

    case_inputs[("AeroDyn15","WakeMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","AFAeroMod")]   = {'vals':[2], 'group':0}
    case_inputs[("AeroDyn15","TwrPotent")]   = {'vals':[0], 'group':0}
    case_inputs[("AeroDyn15","TwrShadow")]   = {'vals':['False'], 'group':0}
    case_inputs[("AeroDyn15","TwrAero")]     = {'vals':['False'], 'group':0}

    case_inputs[("AeroDyn15","SkewMod")]     = {'vals':[1], 'group':0}
    case_inputs[("AeroDyn15","TipLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","HubLoss")]     = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TanInd")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","AIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","TIDrag")]      = {'vals':['True'], 'group':0}
    case_inputs[("AeroDyn15","IndToler")]    = {'vals':[1.e-5], 'group':0}
    case_inputs[("AeroDyn15","MaxIter")]     = {'vals':[5000], 'group':0}
    case_inputs[("AeroDyn15","UseBlCm")]     = {'vals':['True'], 'group':0}
    
    case_list, case_name_list = iec.execute(case_inputs=case_inputs)

    channels  = ["TipDxc1", "TipDyc1", "TipDzc1", "TipDxc2", "TipDyc2", "TipDzc2", "TipDxc3", "TipDyc3", "TipDzc3"]
    channels += ["RootMxc1", "RootMyc1", "RootMzc1", "RootMxc2", "RootMyc2", "RootMzc2", "RootMxc3", "RootMyc3", "RootMzc3"]
    channels += ["RootFxc1", "RootFyc1", "RootFzc1", "RootFxc2", "RootFyc2", "RootFzc2", "RootFxc3", "RootFyc3", "RootFzc3"]
    channels += ["RtAeroCp", "RotTorq", "RotThrust", "RotSpeed", "NacYaw"]

    channels += ["B1N1Fx", "B1N2Fx", "B1N3Fx", "B1N4Fx", "B1N5Fx", "B1N6Fx", "B1N7Fx", "B1N8Fx", "B1N9Fx"]
    channels += ["B1N1Fy", "B1N2Fy", "B1N3Fy", "B1N4Fy", "B1N5Fy", "B1N6Fy", "B1N7Fy", "B1N8Fy", "B1N9Fy"]

    return case_list, case_name_list, channels


if __name__ == "__main__":

    # power_curve()

    case_list, case_name_list = RotorSE_rated('test', 60., 11., 12.1, 0.)





import numpy as np
import numpy.testing as npt
import unittest
import wisdem.commonse.environment as env
from wisdem.commonse import gravity as g

npts = 100
myones = np.ones((npts,))

class TestPowerWind(unittest.TestCase):
    def setUp(self):
        self.params = {}
        self.unknowns = {}
        self.resid = None

        self.params['shearExp'] = 2.0
        self.params['Uref'] = 5.0
        self.params['zref'] = 3.0
        self.params['z0'] = 0.0
        self.params['z'] = 9.0 * myones
        
        self.wind = env.PowerWind(nPoints=npts)

    def testRegular(self):
        self.wind.compute(self.params, self.unknowns)
        expect = 45.0*myones
        npt.assert_equal(self.unknowns['U'], expect)

    def testIndex(self):
        self.params['z'][1:] = -1.0
        self.wind.compute(self.params, self.unknowns)
        expect = 45.0*myones
        expect[1:] = 0.0
        npt.assert_equal(self.unknowns['U'], expect)

    def testZ0(self):
        self.params['z0'] = 10.0
        self.params['z'] += 10.0
        self.params['zref'] += 10.0
        self.wind.compute(self.params, self.unknowns)
        expect = 45.0*myones
        npt.assert_equal(self.unknowns['U'], expect)


class TestLinearWaves(unittest.TestCase):
    def setUp(self):
        self.params = {}
        self.unknowns = {}
        self.resid = None

        self.params['rho'] = 1e3
        self.params['hmax'] = 2.0
        self.params['Uc'] = 5.0
        self.params['z_floor'] = -30.0
        self.params['z_surface'] = 0.0
        self.params['z'] = -2.0 * myones

        self.wave = env.LinearWaves(nPoints=npts)
        
    def testRegular(self):
        D = np.abs(self.params['z_floor'])
        k = 2.5
        omega = np.sqrt(g*k*np.tanh(k*D))
        self.params['T'] = 2.0 * np.pi / omega
        
        self.wave.compute(self.params, self.unknowns)
        a = 1.0 #0.5*hmax
        z = -2.0
        rho = 1e3
        U_exp = 5 + omega*a*np.cosh(k*(z+D))/np.sinh(k*D)
        W_exp = -omega*a*np.sinh(k*(z+D))/np.sinh(k*D)
        V_exp = np.sqrt(U_exp**2 + W_exp**2)
        A_exp = omega*omega*a*np.cosh(k*(z+D))/np.sinh(k*D)
        p_exp = -rho*g*(z - a*np.cosh(k*(z+D))/np.cosh(k*D))
        
        npt.assert_almost_equal(self.unknowns['U'], U_exp)
        npt.assert_almost_equal(self.unknowns['W'], W_exp)
        npt.assert_almost_equal(self.unknowns['V'], V_exp)
        npt.assert_almost_equal(self.unknowns['A'], A_exp)
        npt.assert_almost_equal(self.unknowns['p'], p_exp)

        # Positive depth input
        self.params['z_floor'] = 30.0
        self.wave.compute(self.params, self.unknowns)
        npt.assert_almost_equal(self.unknowns['U'], U_exp)
        npt.assert_almost_equal(self.unknowns['W'], W_exp)
        npt.assert_almost_equal(self.unknowns['V'], V_exp)
        npt.assert_almost_equal(self.unknowns['A'], A_exp)
        npt.assert_almost_equal(self.unknowns['p'], p_exp)

    def testPositiveZ(self):
        self.params['T'] = 2.0 
        self.params['z'] = 2.0 * myones
        self.wave.compute(self.params, self.unknowns)
        npt.assert_equal(self.unknowns['U'], 0.0)
        npt.assert_equal(self.unknowns['W'], 0.0)
        npt.assert_equal(self.unknowns['V'], 0.0)
        npt.assert_equal(self.unknowns['A'], 0.0)
        npt.assert_equal(self.unknowns['p'], 0.0)

        
    def testQuiet(self):
        self.params['hmax'] = 0.0 
        self.params['T'] = 2.0
        self.wave.compute(self.params, self.unknowns)
        p_exp = 2e3*g
        npt.assert_equal(self.unknowns['U'], 5.0)
        npt.assert_equal(self.unknowns['W'], 0.0)
        npt.assert_equal(self.unknowns['V'], 5.0)
        npt.assert_equal(self.unknowns['A'], 0.0)
        npt.assert_equal(self.unknowns['p'], p_exp)
        
        
def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPowerWind))
    suite.addTest(unittest.makeSuite(TestLinearWaves))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())



import numpy as np
from math import gamma
from openmdao.api import ExplicitComponent

# ---------------------
# Map Design Variables to Discretization
# ---------------------

class PDFBase(ExplicitComponent):
    """probability distribution function"""
    def initialize(self):
        self.options.declare('nspline')
    def setup(self):
        nspline = self.options['nspline']
        self.add_input('x', shape=nspline)

        self.add_output('f', shape=nspline)


class CDFBase(ExplicitComponent):
    """cumulative distribution function"""
    def initialize(self):
        self.options.declare('nspline')
    def setup(self):
        nspline = self.options['nspline']

        self.add_input('x', shape=nspline,  units='m/s', desc='corresponding reference height')
        self.add_input('k', shape=1, desc='shape or form factor')

        self.add_output('F', shape=nspline, units='m/s', desc='magnitude of wind speed at each z location')


class WeibullCDF(CDFBase):
    def setup(self):
        super(WeibullCDF, self).setup()
        """Weibull cumulative distribution function"""

        self.add_input('A', shape=1, desc='scale factor')
        
        self.declare_partials('F', 'x')

    def compute(self, inputs, outputs):
        outputs['F'] = 1.0 - np.exp(-(inputs['x']/inputs['A'])**inputs['k'])

    def compute_partials(self, inputs, J):

        x = inputs['x']
        A = inputs['A']
        k = inputs['k']
        
        J['F', 'x'] = np.diag(np.exp(-(x/A)**k)*(x/A)**(k-1)*k/A)
        


class WeibullWithMeanCDF(CDFBase):
    def setup(self):
        super(WeibullWithMeanCDF, self).setup()
        """Weibull cumulative distribution function"""

        self.add_input('xbar', shape=1, units='m/s', desc='mean value of distribution')

        self.declare_partials('F', 'x')
        self.declare_partials('F', 'xbar')
        
    def compute(self, inputs, outputs):

        A = inputs['xbar'] / gamma(1.0 + 1.0/inputs['k'])

        outputs['F'] = 1.0 - np.exp(-(inputs['x']/A)**inputs['k'])


    def compute_partials(self, inputs, J):

        x = inputs['x']
        k = inputs['k']
        A = inputs['xbar'] / gamma(1.0 + 1.0/k)
        dx = np.diag(np.exp(-(x/A)**k)*(x/A)**(k-1)*k/A)
        dxbar = -np.exp(-(x/A)**k)*(x/A)**(k-1)*k*x/A**2/gamma(1.0 + 1.0/k)
        
        J['F', 'x'] = dx
        J['F', 'xbar'] = dxbar

        

class RayleighCDF(CDFBase):
    def setup(self):
        super(RayleighCDF,  self).setup()

        # variables
        self.add_input('xbar', shape=1, units='m/s', desc='reference wind speed (usually at hub height)')

        self.declare_partials('F', 'x')
        self.declare_partials('F', 'xbar')

    def compute(self, inputs, outputs):

        outputs['F'] = 1.0 - np.exp(-np.pi/4.0*(inputs['x']/inputs['xbar'])**2)

    def compute_partials(self, inputs, J):

        x = inputs['x']
        xbar = inputs['xbar']
        dx = np.diag(np.exp(-np.pi/4.0*(x/xbar)**2)*np.pi*x/(2.0*xbar**2))
        dxbar = -np.exp(-np.pi/4.0*(x/xbar)**2)*np.pi*x**2/(2.0*xbar**3)
        
        J['F', 'x'] = dx
        J['F', 'xbar'] = dxbar
        
        




from __future__ import print_function
import numpy as np
import openmdao.api as om
from wisdem.rotorse.rotor import RotorSE
from wisdem.towerse.tower import TowerSE
from wisdem.commonse.turbine_constraints import TurbineConstraints
from wisdem.turbine_costsse.turbine_costsse_2015 import Turbine_CostsSE_2015
from wisdem.plant_financese.plant_finance import PlantFinance
from wisdem.drivetrainse.rna  import RNA
from wisdem.orbit.api.wisdem.fixed import Orbit
from wisdem.commonse.mpi_tools import MPI

# np.seterr(all ='raise')
        
# Group to link the openmdao components
class MonopileTurbine(om.Group):

    def initialize(self):
        self.options.declare('RefBlade')
        self.options.declare('FASTpref', default={})
        self.options.declare('Nsection_Tow', default = 6)
        self.options.declare('VerbosityCosts', default = True)
        self.options.declare('user_update_routine',     default=None)
        
        
    def setup(self):
        
        RefBlade            = self.options['RefBlade']
        Nsection_Tow        = self.options['Nsection_Tow']        
        user_update_routine = self.options['user_update_routine']
        if 'Analysis_Level' in self.options['FASTpref']:
            Analysis_Level  = self.options['FASTpref']['Analysis_Level']
        else:
            Analysis_Level  = 0
        
        # Define all input variables from all models
        myIndeps = om.IndepVarComp()
        myIndeps.add_discrete_output('crane',    False)

        # Turbine Costs
        myIndeps.add_discrete_output('bearing_number', 0)
        
        # Tower and Frame3DD options
        myIndeps.add_output('project_lifetime',             0.0, units='yr')
        myIndeps.add_output('max_taper_ratio',              0.0)
        myIndeps.add_output('min_diameter_thickness_ratio', 0.0)

        # Environment
        myIndeps.add_output('wind_bottom_height',   0.0, units='m')
        myIndeps.add_output('wind_beta',            0.0, units='deg')
        myIndeps.add_output('cd_usr', -1.)

        # Environment Offshore
        myIndeps.add_output('offshore',           True)
        myIndeps.add_output('water_depth',        0.0, units='m')

        # Design standards
        myIndeps.add_output('gamma_b', 0.0)
        myIndeps.add_output('gamma_n', 0.0)

        # Column
        myIndeps.add_output('morison_mass_coefficient', 2.0)
        myIndeps.add_output('material_density',         0.0, units='kg/m**3')
        myIndeps.add_output('E',                        0.0, units='N/m**2')
        myIndeps.add_output('yield_stress',             0.0, units='N/m**2')

        # Pontoons
        myIndeps.add_output('G', 0.0, units='N/m**2')
        
        # LCOE
        myIndeps.add_output('labor_cost_rate',      0.0, units='USD/min')
        myIndeps.add_output('material_cost_rate',   0.0, units='USD/kg')
        myIndeps.add_output('painting_cost_rate',   0.0, units='USD/m**2')
        myIndeps.add_discrete_output('number_of_turbines', 0)
        myIndeps.add_output('annual_opex',          0.0, units='USD/kW/yr') # TODO: Replace with output connection
        myIndeps.add_output('fixed_charge_rate',    0.0)
        myIndeps.add_output('wake_loss_factor',     0.0)
        
        myIndeps.add_output('overhang', 0.0, units='m')
        myIndeps.add_output('hub_cm', np.zeros(3), units='m')
        myIndeps.add_output('nac_cm', np.zeros(3), units='m')
        myIndeps.add_output('hub_I', np.zeros(6), units='kg*m**2')
        myIndeps.add_output('nac_I', np.zeros(6), units='kg*m**2')
        myIndeps.add_output('hub_mass', 0.0, units='kg')
        myIndeps.add_output('nac_mass', 0.0, units='kg')
        myIndeps.add_output('hss_mass', 0.0, units='kg')
        myIndeps.add_output('lss_mass', 0.0, units='kg')
        myIndeps.add_output('cover_mass', 0.0, units='kg')
        myIndeps.add_output('pitch_system_mass', 0.0, units='kg')
        myIndeps.add_output('platforms_mass', 0.0, units='kg')
        myIndeps.add_output('spinner_mass', 0.0, units='kg')
        myIndeps.add_output('transformer_mass', 0.0, units='kg')
        myIndeps.add_output('vs_electronics_mass', 0.0, units='kg')
        myIndeps.add_output('yaw_mass', 0.0, units='kg')
        myIndeps.add_output('gearbox_mass', 0.0, units='kg')
        myIndeps.add_output('generator_mass', 0.0, units='kg')
        myIndeps.add_output('bedplate_mass', 0.0, units='kg')
        myIndeps.add_output('main_bearing_mass', 0.0, units='kg')

        self.add_subsystem('myIndeps', myIndeps, promotes=['*'])

        
        # Add components
        self.add_subsystem('rotorse', RotorSE(RefBlade=RefBlade,
                                              npts_coarse_power_curve=20,
                                              npts_spline_power_curve=200,
                                              regulation_reg_II5=True,
                                              regulation_reg_III=True,
                                              Analysis_Level=Analysis_Level,
                                              FASTpref=self.options['FASTpref'],
                                              topLevelFlag=True,
                                              user_update_routine=user_update_routine), promotes=['*'])
        
        self.add_subsystem('rna', RNA(nLC=1), promotes=['hub_mass','nac_mass','nac_cm','hub_cm','tilt'])
        
        # Tower and substructure
        self.add_subsystem('tow',TowerSE(nLC=1,
                                         nPoints=Nsection_Tow+1,
                                         nFull=5*Nsection_Tow+1,
                                         wind='PowerWind',
                                         topLevelFlag=False,
                                         monopile=True),
                           promotes=['water_density','water_viscosity','wave_beta',
                                     'significant_wave_height','significant_wave_period',
                                     'material_density','E','G','tower_section_height',
                                     'tower_wall_thickness', 'tower_outer_diameter',
                                     'tower_outfitting_factor','tower_buckling_length',
                                     'transition_piece_mass','transition_piece_height',
                                     'max_taper','min_d_to_t','rna_mass','rna_cg','rna_I',
                                     'tower_add_gravity','tower_mass','tower_I_base','hub_height',
                                     'foundation_height','soil_G','soil_nu','suctionpile_depth','suctionpile_depth_diam_ratio',
                                     'monopile_mass','monopile_cost','monopile_length','tower_raw_cost',
                                     'gamma_f','gamma_m','gamma_b','gamma_n','gamma_fatigue',
                                     'labor_cost_rate','material_cost_rate','painting_cost_rate','z_full','d_full','t_full',
                                     'DC','shear','geom','tower_force_discretization','nM','Mmethod','lump','tol','shift'])

        # Turbine constraints
        self.add_subsystem('tcons', TurbineConstraints(nFull=5*(Nsection_Tow+1)+1), promotes=['*'])
        
        # Turbine costs
        self.add_subsystem('tcost', Turbine_CostsSE_2015(verbosity=self.options['VerbosityCosts'], topLevelFlag=False), promotes=['*'])

        # Use ORBIT for BOS costs
        self.add_subsystem('orbit', Orbit(), promotes=['wtiv', 'feeder', 'num_feeders', 'oss_install_vessel',
                                                       'hub_height',
                                                       'number_of_turbines',
                                                       'tower_mass',
                                                       'monopile_length',
                                                       'monopile_mass',
                                                       'transition_piece_mass',
                                                       'site_distance',
                                                       'site_distance_to_landfall',
                                                       'site_distance_to_interconnection',
                                                       'plant_turbine_spacing',
                                                       'plant_row_spacing',
                                                       'plant_substation_distance',
                                                       'tower_deck_space',
                                                       'nacelle_deck_space',
                                                       'blade_deck_space',
                                                       'port_cost_per_month',
                                                       'monopile_deck_space',
                                                       'transition_piece_deck_space',
                                                       #'scour_protection_depth',
                                                       'commissioning_pct',
                                                       'decommissioning_pct'])
        
        # LCOE Calculation
        self.add_subsystem('plantfinancese', PlantFinance(verbosity=self.options['VerbosityCosts']), promotes=['machine_rating','lcoe',
                                                                                                               'fixed_charge_rate','wake_loss_factor'])
        
    
        # Set up connections        

        # Connections to DriveSE
        self.connect('Fxyz_total',      'rna.loads.F')
        self.connect('Mxyz_total',      'rna.loads.M')
        self.connect('mass_all_blades',  'rna.blades_mass')
        self.connect('I_all_blades',        'rna.blades_I')

        self.connect('material_density', 'tow.tower.rho')

        # Connections to TowerSE
        self.connect('rna.loads.top_F',         'tow.pre.rna_F')
        self.connect('rna.loads.top_M',         'tow.pre.rna_M')
        self.connect('rna.rna_I_TT',            ['rna_I','tow.pre.mI'])
        self.connect('rna.rna_cm',              ['rna_cg','tow.pre.mrho'])
        self.connect('rna.rna_mass',            ['rna_mass','tow.pre.mass'])
        self.connect('rs.gust.V_gust',          'tow.wind.Uref')
        self.connect('wind_reference_height',   ['tow.wind.zref','wind.zref'])
        # self.connect('wind_bottom_height',      ['tow.wind.z0','tow.wave.z_surface', 'wind.z0'])  # offshore
        self.connect('wind_bottom_height',      ['tow.wind.z0', 'wind.z0'])
        self.connect('shearExp',                ['tow.wind.shearExp'])
        # self.connect('morison_mass_coefficient','tow.cm')                                         # offshore
        self.connect('yield_stress',            'tow.sigma_y')
        self.connect('max_taper_ratio',         'max_taper')
        self.connect('min_diameter_thickness_ratio', 'min_d_to_t')

        self.connect('rho',         'tow.windLoads.rho')
        self.connect('mu',          'tow.windLoads.mu')
        self.connect('wind_beta',   'tow.windLoads.beta')
        
        # Connections to TurbineConstraints
        self.connect('nBlades',                 'blade_number')
        self.connect('control_maxOmega',        'rotor_omega')
        self.connect('tow.post.structural_frequencies', 'tower_freq')        
                
        # Connections to TurbineCostSE
        self.connect('mass_one_blade',              ['blade_mass','orbit.blade_mass'])
        self.connect('total_blade_cost',            'blade_cost_external')
        self.connect('tower_raw_cost',          'tower_cost_external')

        # Connections to ORBIT
        self.connect('water_depth', 'orbit.site_depth')
        self.connect('rs.gust.V_gust', 'orbit.site_mean_windspeed')
        self.connect('machine_rating', 'orbit.turbine_rating')
        self.connect('rated_V', 'orbit.turbine_rated_windspeed')
        self.connect('turbine_cost_kW', 'orbit.turbine_capex')
        self.connect('diameter', 'orbit.turbine_rotor_diameter')
        self.connect('nac_mass', 'orbit.nacelle_mass')
        self.connect('tower_outer_diameter', 'orbit.monopile_diameter', src_indices=[0])
        
        # Connections to PlantFinanceSE
        self.connect('AEP',                 'plantfinancese.turbine_aep')
        self.connect('turbine_cost_kW',     'plantfinancese.tcc_per_kW')
        self.connect('number_of_turbines',  'plantfinancese.turbine_number')
        self.connect('orbit.total_capex_kW','plantfinancese.bos_per_kW')
        self.connect('annual_opex',         'plantfinancese.opex_per_kW')
    

    





    



from floating_instance import FloatingInstance
from commonse.utilities import sectional2nodal
import numpy as np


class SparInstance(FloatingInstance):
    def __init__(self):
        super(SparInstance, self).__init__()

        # Parameters beyond those in superclass
        self.params['number_of_offset_columns'] = 0
        self.params['cross_attachment_pontoons_int'] = 0
        self.params['lower_attachment_pontoons_int'] = 0
        self.params['upper_attachment_pontoons_int'] = 0
        self.params['lower_ring_pontoons_int'] = 0
        self.params['upper_ring_pontoons_int'] = 0
        self.params['outer_cross_pontoons_int'] = 0
 
        # Typically design (OC3)
        self.params['main_freeboard'] = 10.0
        self.params['fairlead_location'] = 0.75862 # Want 5m
        self.set_length_main(130.0)
        self.params['main_section_height'] = np.array([36.0, 72.0, 8.0, 14.0])
        self.params['main_outer_diameter'] = 2*np.array([4.7, 4.7, 4.7, 3.25, 3.25])
        self.params['main_wall_thickness'] = 0.05
        self.params['fairlead_offset_from_shell'] = 5.2-4.7
        self.params['main_permanent_ballast_height'] = 10.0
        
        # OC3
        self.params['water_depth'] = 320.0
        self.params['Hs'] = 10.8
        self.params['T'] = 9.8
        self.params['Uref'] = 11.0
        self.params['zref'] = 119.0
        self.params['shearExp'] = 0.11
        self.params['cm'] = 2.0

        self.params['number_of_mooring_connections']        = 3
        self.params['mooring_lines_per_connection']         = 1
        self.params['mooring_line_length'] = 902.2
        self.params['anchor_radius'] = 853.87
        self.params['mooring_diameter'] = 0.09
        
        # Change scalars to vectors where needed
        self.check_vectors()
        

    def get_constraints(self):
        conlist = super(SparInstance, self).get_constraints()

        poplist = []
        for k in range(len(conlist)):
            if ( (conlist[k][0].find('off') >= 0) or
                 (conlist[k][0].find('pontoon') >= 0) or
                 (conlist[k][0].find('main_connection_ratio') >= 0) ):
                poplist.append(k)

        poplist.reverse()
        for k in poplist: conlist.pop(k)

        return conlist

    
    def visualize(self, fname=None):
        fig = self.init_figure()

        self.draw_ocean(fig)

        self.draw_mooring(fig, self.prob['mooring_plot_matrix'])

        zcut = 1.0 + self.params['main_freeboard']
        self.draw_pontoons(fig, self.prob['plot_matrix'], 0.5*self.params['fairlead_support_outer_diameter'], zcut)

        self.draw_column(fig, [0.0, 0.0], self.params['main_freeboard'], self.params['main_section_height'],
                           0.5*self.params['main_outer_diameter'], self.params['main_stiffener_spacing'])

        t_full = sectional2nodal(self.params['main_wall_thickness'])
        self.draw_ballast(fig, [0.0, 0.0], self.params['main_freeboard'], self.params['main_section_height'],
                          0.5*self.params['main_outer_diameter']-t_full,
                          self.params['main_permanent_ballast_height'], self.prob['variable_ballast_height'])

        self.draw_column(fig, [0.0, 0.0], self.params['hub_height'], self.params['tower_section_height'],
                         0.5*self.params['tower_outer_diameter'], None, (0.9,)*3)

        if self.prob['main.buoyancy_tank_mass'] > 0.0:
            self.draw_buoyancy_tank(fig, [0.0, 0.0], self.params['main_freeboard'],
                                        self.params['main_section_height'],
                                        self.params['main_buoyancy_tank_location'],
                                        0.5*self.params['main_buoyancy_tank_diameter'],
                                        self.params['main_buoyancy_tank_height'])
        
        self.set_figure(fig, fname)



"""generator.py
Created by Latha Sethuraman, Katherine Dykes. 
Copyright (c) NREL. All rights reserved.

Electromagnetic design based on conventional magnetic circuit laws
Structural design based on McDonald's thesis """

import openmdao.api as om
from om import Group, Problem, ExplicitComponent, ExecComp, IndepVarComp
#from openmdao.api import ScipyOptimizeDriver, pyOptSparseDriver
#from openmdao.drivers.pyoptsparse_driver import pyOptSparseDriver

import numpy as np
#from scig import SCIG
#from dfig import DFIG
#from eesg import EESG
#from pmsg_arms import PMSG_Arms
#from pmsg_disc import PMSG_Disc

from scigOM import SCIG_OM
from dfigOM import DFIG_OM
from eesgOM import EESG_OM
from pmsg_armsOM import PMSG_Arms_OM
from pmsg_discOM import PMSG_Disc_OM


class Generator_Cost(ExplicitComponent):
    """ Provides a material cost estimate for a PMSG _arms generator. Manufacturing costs are excluded"""
    def setup(self):
        
        # Specific cost of material by type
        self.add_input('C_Cu',  val=0.0, units='USD/kg', desc='Specific cost of copper')
        self.add_input('C_Fe',  val=0.0, units='USD/kg', desc='Specific cost of magnetic steel/iron')
        self.add_input('C_Fes', val=0.0, units='USD/kg', desc='Specific cost of structural steel')
        self.add_input('C_PM',  val=0.0, units='USD/kg', desc='Specific cost of Magnet')  
        
        # Mass of each material type
        self.add_input('Copper',          val=0.0, units='kg', desc='Copper mass')
        self.add_input('Iron',            val=0.0, units='kg', desc='Iron mass')
        self.add_input('mass_PM' ,        val=0.0, units='kg', desc='Magnet mass')
        self.add_input('Structural_mass', val=0.0, units='kg', desc='Structural mass')
        
        # Outputs
        self.add_output('Costs', val=0.0, units='USD', desc='Total cost')

        #self.declare_partials('*', '*', method='fd', form='central', step=1e-6)
        
    def compute(self, inputs, outputs):
        Copper          = inputs['Copper']
        Iron            = inputs['Iron']
        mass_PM         = inputs['mass_PM']
        Structural_mass = inputs['Structural_mass']
        C_Cu            = inputs['C_Cu']
        C_Fes           = inputs['C_Fes']
        C_Fe            = inputs['C_Fe']
        C_PM            = inputs['C_PM']
                
        # Material cost as a function of material mass and specific cost of material
        K_gen            = Copper*C_Cu + Iron*C_Fe + C_PM*mass_PM #%M_pm*K_pm; # 
        Cost_str         = C_Fes*Structural_mass
        outputs['Costs'] = K_gen + Cost_str
        

        

class Generator(Group):

    def initialize(self):
        genTypes = ['scig','dfig','eesg','pmsg_arms','pmsg_disc']
        self.options.declare('topLevelFlag', default=True)
        self.options.declare('design', values=genTypes + [m.upper() for m in genTypes])
    
    def setup(self):
        topLevelFlag = self.options['topLevelFlag']
        genType      = self.options['design']
        
        generatorIndeps = IndepVarComp()
        generatorIndeps.add_output('shaft_cm', val=np.zeros(3), units='m')
        generatorIndeps.add_output('shaft_length', val=0.0, units='m')
        
        generatorIndeps.add_output('Gearbox_efficiency', val=0.0)
        generatorIndeps.add_output('r_s',   val=0.0, units='m')
        generatorIndeps.add_output('len_s', val=0.0, units='m')
        generatorIndeps.add_output('h_s',   val=0.0, units='m')
        generatorIndeps.add_output('h_r',   val=0.0, units='m')
        generatorIndeps.add_output('h_m',   val=0.0, units='m')
        generatorIndeps.add_output('S_Nmax', val=0.0, units='T')
        generatorIndeps.add_output('I_0',   val=0.0, units='A')

        if genType.lower() in ['eesg','pmsg_arms','pmsg_disc']:
            generatorIndeps.add_output('tau_p', val=0.0, units='m')
            generatorIndeps.add_output('h_ys',  val=0.0, units='m')
            generatorIndeps.add_output('h_yr',  val=0.0, units='m')
            generatorIndeps.add_output('b_r',   val=0.0, units='m')
        elif genType.lower() in ['scig','dfig']:
            generatorIndeps.add_output('B_symax', val=0.0)
            
        generatorIndeps.add_output('I_f',  val=0.0, units='A')
        generatorIndeps.add_output('N_f',  val=0.0, units='A')
        generatorIndeps.add_output('n_s',  val=0.0)
        generatorIndeps.add_output('b_st', val=0.0, units='m')
        generatorIndeps.add_output('n_r',  val=0.0)
        generatorIndeps.add_output('d_r',  val=0.0, units='m')
        generatorIndeps.add_output('d_s',  val=0.0, units='m')
        generatorIndeps.add_output('t_d',  val=0.0, units='m')
        generatorIndeps.add_output('t_wr', val=0.0, units='m')
        generatorIndeps.add_output('t_ws', val=0.0, units='m')
        generatorIndeps.add_output('R_o',  val=0.0, units='m')
        
        generatorIndeps.add_output('rho_Fes',    val=0.0, units='kg/m**3')
        generatorIndeps.add_output('rho_Fe',     val=0.0, units='kg/m**3')
        generatorIndeps.add_output('rho_Copper', val=0.0, units='kg/m**3')
        generatorIndeps.add_output('rho_PM',     val=0.0, units='kg/m**3')
        
        generatorIndeps.add_output('C_Cu',  val=0.0, units='USD/kg')
        generatorIndeps.add_output('C_Fe',  val=0.0, units='USD/kg')
        generatorIndeps.add_output('C_Fes', val=0.0, units='USD/kg')
        generatorIndeps.add_output('C_PM',  val=0.0, units='USD/kg')
        
        self.add_subsystem('generatorIndeps', generatorIndeps, promotes=['*'])

        if topLevelFlag:
            sharedIndeps = IndepVarComp()
            sharedIndeps.add_output('machine_rating', 0.0, units='W')
            sharedIndeps.add_output('n_nom', 0.0, units='rpm')
            sharedIndeps.add_output('Torque', 0.0, units='N*m')
            self.add_subsystem('sharedIndeps', sharedIndeps, promotes=['*'])
        
        # Add generator design component and cost
        if genType.lower() == 'scig':
            mygen = SCIG_OM
            
        elif genType.lower() == 'dfig':
            mygen = DFIG_OM
            
        elif genType.lower() == 'eesg':
            mygen = EESG_OM
            
        elif genType.lower() == 'pmsg_arms':
            mygen = PMSG_Arms_OM
            
        elif genType.lower() == 'pmsg_disc':
            mygen = PMSG_Disc_OM
            
        self.add_subsystem('generator', mygen(), promotes=['*'])
        self.add_subsystem('gen_cost', Generator_Cost(), promotes=['*'])

        # Now add constraints
        if genType.lower() in ['eesg', 'pmsg_arms', 'pmsg_disc']:
            self.add_subsystem('con_uAs', ExecComp('con_uAs =u_all_s-u_As'), promotes=['*'])
            self.add_subsystem('con_zAs', ExecComp('con_zAs =z_all_s-z_A_s'), promotes=['*'])
            self.add_subsystem('con_yAs', ExecComp('con_yAs =y_all-y_As'), promotes=['*'])
            self.add_subsystem('con_bst', ExecComp('con_bst =b_all_s-b_st',
                                                   b_st={'units':'m'}), promotes=['*'])
            self.add_subsystem('con_uAr', ExecComp('con_uAr =u_all_r-u_Ar'), promotes=['*'])
            self.add_subsystem('con_yAr', ExecComp('con_yAr =y_all-y_Ar'), promotes=['*'])
            self.add_subsystem('con_TC2', ExecComp('con_TC2 =TC2-TC1'), promotes=['*'])
            self.add_subsystem('con_TC3', ExecComp('con_TC3 =TC3-TC1'), promotes=['*'])

        if genType.lower() in ['pmsg_arms', 'pmsg_disc']:
            self.add_subsystem('con_Bsmax', ExecComp('con_Bsmax =B_g-B_smax'), promotes=['*'])
            
        if genType.lower() in ['eesg', 'pmsg_arms']:
            self.add_subsystem('con_zAr', ExecComp('con_zAr =z_all_r-z_A_r'), promotes=['*'])
            self.add_subsystem('con_br', ExecComp('con_br =b_all_r-b_r',
                                                  b_r={'units':'m'}), promotes=['*'])
            
        if genType.lower() in ['scig','dfig']:
            self.add_subsystem('TC', ExecComp('TC = TC2-TC1'), promotes=['*'])
            self.add_subsystem('K_rad_L',ExecComp('K_rad_L = K_rad-K_rad_LL'), promotes=['*'])
            self.add_subsystem('K_rad_U',ExecComp('K_rad_U = K_rad-K_rad_UL'), promotes=['*'])
            self.add_subsystem('D_ratio_L',ExecComp('D_ratio_L = D_ratio-D_ratio_LL'), promotes=['*'])
            self.add_subsystem('D_ratio_U',ExecComp('D_ratio_U = D_ratio-D_ratio_UL'), promotes=['*'])
            

def optimization_example(genType, exportFlag=False):
    genType = genType.lower()
    
    #Example optimization of a generator for costs on a 5 MW reference turbine
    opt_problem=Problem()
    opt_problem.model = Generator(design=genType, topLevelFlag=True)
    
    # add optimizer and set-up problem (using user defined input on objective function)
    '''
    opt_problem.driver = om.pyOptSparseDriver() #ScipyOptimizeDriver()
    opt_problem.driver.options['optimizer'] = 'CONMIN'
    opt_problem.driver.opt_settings['IPRINT'] = 4
    opt_problem.driver.opt_settings['ITRM'] = 3
    opt_problem.driver.opt_settings['ITMAX'] = 10
    opt_problem.driver.opt_settings['DELFUN'] = 1e-3
    opt_problem.driver.opt_settings['DABFUN'] = 1e-3
    opt_problem.driver.opt_settings['IFILE'] = 'CONMIN_'+genType.upper()+'.out'
    '''
    opt_problem.driver = om.ScipyOptimizeDriver()
    opt_problem.driver.options['optimizer'] = 'SLSQP'
    
    # Specificiency target efficiency(%)
    Eta_Target = 93.0

    eps = 1e-6
    
    # Set up design variables and bounds for a SCIG designed for a 5MW turbine
    if genType in ['dfig', 'scig']:
        # Design variables
        opt_problem.model.add_design_var('r_s',     lower=0.2,  upper=1.0)
        opt_problem.model.add_design_var('len_s',   lower=0.4,  upper=2.0)
        opt_problem.model.add_design_var('h_s',     lower=0.04, upper=0.1)
        opt_problem.model.add_design_var('h_r',     lower=0.04, upper=0.1)
        opt_problem.model.add_design_var('B_symax', lower=1.0,  upper=2.0-eps)

        # Constraints
        opt_problem.model.add_constraint('Overall_eff',        lower=Eta_Target)
        opt_problem.model.add_constraint('E_p',                lower=500.0+eps, upper=5000.0-eps)
        opt_problem.model.add_constraint('TC',                 lower=0.0+eps)
        opt_problem.model.add_constraint('B_g',                lower=0.7,       upper=1.2)
        opt_problem.model.add_constraint('B_trmax',                             upper=2.0-eps)
        opt_problem.model.add_constraint('B_tsmax',                             upper=2.0-eps)
        opt_problem.model.add_constraint('A_1',                                 upper=60000.0-eps)
        opt_problem.model.add_constraint('J_s',                                 upper=6.0)
        opt_problem.model.add_constraint('J_r',                                 upper=6.0)
        opt_problem.model.add_constraint('Slot_aspect_ratio1', lower=4.0,       upper=10.0)

    if genType == 'scig':
        opt_problem.model.add_design_var('I_0',       lower=5.0, upper=200.0)
        opt_problem.model.add_constraint('B_rymax',   upper=2.0-eps)
        opt_problem.model.add_constraint('K_rad_L',   lower=0.0)
        opt_problem.model.add_constraint('K_rad_U',   upper=0.0)
        opt_problem.model.add_constraint('D_ratio_L', lower=0.0)
        opt_problem.model.add_constraint('D_ratio_U', upper=0.0)
        
    if genType == 'dfig':
        opt_problem.model.add_design_var('I_0',          lower=5.,   upper=100.0)
        opt_problem.model.add_design_var('S_Nmax',       lower=-0.3, upper=-0.1)
        opt_problem.model.add_constraint('K_rad',        lower=0.2,  upper=1.5)
        opt_problem.model.add_constraint('D_ratio',      lower=1.37, upper=1.4)
        opt_problem.model.add_constraint('Current_ratio',lower=0.1,  upper=0.3)

    if genType == 'eesg':
        # Design variables
        opt_problem.model.add_design_var('r_s',    lower=0.5,   upper=9.0)
        opt_problem.model.add_design_var('len_s',  lower=0.5,   upper=2.5)
        opt_problem.model.add_design_var('h_s',    lower=0.06,  upper=0.15)
        opt_problem.model.add_design_var('tau_p',  lower=0.04,  upper=0.2)
        opt_problem.model.add_design_var('N_f',    lower=10,    upper=300)
        opt_problem.model.add_design_var('I_f',    lower=1,     upper=500)
        opt_problem.model.add_design_var('n_r',    lower=5.0,   upper=15.0)
        opt_problem.model.add_design_var('h_yr',   lower=0.01,  upper=0.25)
        opt_problem.model.add_design_var('h_ys',   lower=0.01,  upper=0.25)
        opt_problem.model.add_design_var('b_r',    lower=0.1,   upper=1.5)
        opt_problem.model.add_design_var('d_r',    lower=0.1,   upper=1.5)
        opt_problem.model.add_design_var('t_wr',   lower=0.001, upper=0.2)
        opt_problem.model.add_design_var('n_s',    lower=5.0,   upper=15.0)
        opt_problem.model.add_design_var('b_st',   lower=0.1,   upper=1.5)
        opt_problem.model.add_design_var('d_s',    lower=0.1,   upper=1.5)
        opt_problem.model.add_design_var('t_ws',   lower=0.001, upper=0.2)

        # Constraints
        opt_problem.model.add_constraint('B_gfm',       lower=0.617031, upper=1.057768)
        opt_problem.model.add_constraint('B_pc',        upper=2.0)
        opt_problem.model.add_constraint('E_s',         lower=500.0, upper=5000.0)
        opt_problem.model.add_constraint('J_f',         upper=6.0)
        opt_problem.model.add_constraint('n_brushes',   upper=6)
        opt_problem.model.add_constraint('Power_ratio', upper=2-eps)
        
    if genType in ['pmsg_arms','pmsg_disc']:
        # Design variables
        opt_problem.model.add_design_var('r_s',   lower=0.5,   upper=9.0)
        opt_problem.model.add_design_var('len_s', lower=0.5,   upper=2.5)
        opt_problem.model.add_design_var('h_s',   lower=0.04,  upper=0.1)
        opt_problem.model.add_design_var('tau_p', lower=0.04,  upper=0.1)
        opt_problem.model.add_design_var('h_m',   lower=0.005, upper=0.1)
        opt_problem.model.add_design_var('n_r',   lower=5.0,   upper=15.0)
        opt_problem.model.add_design_var('h_yr',  lower=0.045, upper=0.25)
        opt_problem.model.add_design_var('h_ys',  lower=0.045, upper=0.25)
        opt_problem.model.add_design_var('n_s',   lower=5.0,   upper=15.0)
        opt_problem.model.add_design_var('b_st',  lower=0.1,   upper=1.5)
        opt_problem.model.add_design_var('d_s',   lower=0.1,   upper=1.5)
        opt_problem.model.add_design_var('t_ws',  lower=0.001, upper=0.2)
    
        opt_problem.model.add_constraint('con_Bsmax', lower=0.0+eps)
        opt_problem.model.add_constraint('E_p', lower=500.0, upper=5000.0)

    if genType == 'pmsg_arms':
        opt_problem.model.add_design_var('b_r',  lower=0.1,   upper=1.5)
        opt_problem.model.add_design_var('d_r',  lower=0.1,   upper=1.5)
        opt_problem.model.add_design_var('t_wr', lower=0.001, upper=0.2)
        
    if genType == 'pmsg_disc':
        opt_problem.model.add_design_var('t_d', lower=0.1, upper=0.25)
        
    if genType in ['eesg', 'pmsg_arms', 'pmsg_disc']:
        opt_problem.model.add_constraint('B_symax', upper=2.0-eps)
        opt_problem.model.add_constraint('B_rymax', upper=2.0-eps)
        opt_problem.model.add_constraint('B_tmax',  upper=2.0-eps)
        opt_problem.model.add_constraint('B_g',     lower=0.7, upper=1.2)
        opt_problem.model.add_constraint('con_uAs', lower=0.0+eps)
        opt_problem.model.add_constraint('con_zAs', lower=0.0+eps)
        opt_problem.model.add_constraint('con_yAs', lower=0.0+eps)
        opt_problem.model.add_constraint('con_uAr', lower=0.0+eps)
        opt_problem.model.add_constraint('con_yAr', lower=0.0+eps)
        opt_problem.model.add_constraint('con_TC2', lower=0.0+eps)
        opt_problem.model.add_constraint('con_TC3', lower=0.0+eps)
        opt_problem.model.add_constraint('con_bst', lower=0.0-eps)
        opt_problem.model.add_constraint('A_1', upper=60000.0-eps)
        opt_problem.model.add_constraint('J_s', upper=6.0)
        opt_problem.model.add_constraint('A_Cuscalc', lower=5.0, upper=300)
        opt_problem.model.add_constraint('K_rad', lower=0.2+eps, upper=0.27)
        opt_problem.model.add_constraint('Slot_aspect_ratio', lower=4.0, upper=10.0)
        opt_problem.model.add_constraint('gen_eff', lower=Eta_Target)

    if genType in ['eesg', 'pmsg_arms']:
        opt_problem.model.add_constraint('con_zAr', lower=0.0+eps)
        opt_problem.model.add_constraint('con_br', lower=0.0+eps)
    
    
    Objective_function = 'Costs'
    opt_problem.model.add_objective(Objective_function, scaler=1e-5)
    opt_problem.setup()
    
    # Specify Target machine parameters
    
    opt_problem['machine_rating'] = 5000000.0
    
    if genType in ['scig', 'dfig']:
        opt_problem['n_nom']              = 1200.0
        opt_problem['Gearbox_efficiency'] = 0.955
    elif genType in ['eesg', 'pmsg_arms','pmsg_disc']:
        opt_problem['Torque']             = 4.143289e6
        opt_problem['n_nom']              = 12.1

        
    if genType == 'scig':
        #opt_problem['r_s']     = 0.55 #0.484689156353 #0.55 #meter
        opt_problem['rad_ag']  = 0.55 #0.484689156353 #0.55 #meter
        opt_problem['len_s']   = 1.30 #1.27480124244 #1.3 #meter
        opt_problem['h_s']     = 0.090 #0.098331868116 # 0.090 #meter
        opt_problem['h_r']     = 0.050 #0.04 # 0.050 #meter
        opt_problem['I_0']     = 140  #139.995232826 #140  #Ampere
        opt_problem['B_symax'] = 1.4 #1.86140258387 #1.4 #Tesla
        
    elif genType == 'dfig':
        #opt_problem['r_s']     = 0.61 #0.493167295965 #0.61 #meter
        opt_problem['rad_ag']  = 0.61 #0.493167295965 #0.61 #meter
        opt_problem['len_s']   = 0.49 #1.06173588215 #0.49 #meter
        opt_problem['h_s']     = 0.08 #0.1 # 0.08 #meter
        opt_problem['h_r']     = 0.1 # 0.0998797703231 #0.1 #meter
        opt_problem['I_0']     = 40.0 # 40.0191207049 #40.0 #Ampere
        opt_problem['B_symax'] = 1.3 #1.59611292026 #1.3 #Tesla
        opt_problem['S_Nmax']  = -0.2 #-0.3 #-0.2

    elif genType == 'eesg':
        # Initial design variables 
        #opt_problem['r_s']     = 3.2
        opt_problem['rad_ag']  = 3.2
        opt_problem['len_s']   = 1.4
        opt_problem['h_s']     = 0.060
        opt_problem['tau_p']   = 0.170
        opt_problem['I_f']     = 69
        opt_problem['N_f']     = 100
        opt_problem['h_ys']    = 0.130
        opt_problem['h_yr']    = 0.120
        opt_problem['n_s']     = 5
        opt_problem['b_st']    = 0.470
        opt_problem['n_r']     = 5
        opt_problem['b_r']     = 0.480
        opt_problem['d_r']     = 0.510
        opt_problem['d_s']     = 0.400
        opt_problem['t_wr']    = 0.140
        opt_problem['t_ws']    = 0.070
        opt_problem['R_o']     = 0.43      #10MW: 0.523950817,#5MW: 0.43, #3MW:0.363882632 #1.5MW: 0.2775  0.75MW: 0.17625

    elif genType == 'pmsg_arms':
        #opt_problem['r_s']     = 3.26
        opt_problem['rad_ag']  = 3.26
        opt_problem['len_s']   = 1.60
        opt_problem['h_s']     = 0.070
        opt_problem['tau_p']   = 0.080
        opt_problem['h_m']     = 0.009
        opt_problem['h_ys']    = 0.075
        opt_problem['h_yr']    = 0.075
        opt_problem['n_s']     = 5.0
        opt_problem['b_st']    = 0.480
        opt_problem['n_r']     = 5.0
        opt_problem['b_r']     = 0.530
        opt_problem['d_r']     = 0.700
        opt_problem['d_s']     = 0.350
        opt_problem['t_wr']    = 0.06
        opt_problem['t_ws']    = 0.06
        opt_problem['R_o']     = 0.43           #0.523950817  #0.43  #0.523950817 #0.17625 #0.2775 #0.363882632 ##0.35 #0.523950817 #0.43 #523950817 #0.43 #0.523950817 #0.523950817 #0.17625 #0.2775 #0.363882632 #0.43 #0.523950817 #0.43

    elif genType == 'pmsg_disc':
        #opt_problem['r_s']     = 3.49 #3.494618182
        opt_problem['rad_ag']  = 3.49 #3.494618182
        opt_problem['len_s']   = 1.5 #1.506103927
        opt_problem['h_s']     = 0.06 #0.06034976
        opt_problem['tau_p']   = 0.07 #0.07541515 
        opt_problem['h_m']     = 0.0105 #0.0090100202 
        opt_problem['h_ys']    = 0.085 #0.084247994 #
        opt_problem['h_yr']    = 0.055 #0.0545789687
        opt_problem['n_s']     = 5.0 #5.0
        opt_problem['b_st']    = 0.460 #0.46381
        opt_problem['t_d']     = 0.105 #0.10 
        opt_problem['d_s']     = 0.350 #0.35031 #
        opt_problem['t_ws']    = 0.150 #=0.14720 #
        opt_problem['R_o']     = 0.43 #0.43

    #----------------- try 15MW PMSG_disc --------------
    #  testing 2019 11 04
    
    if genType in ['eesg', 'pmsg_arms','pmsg_disc']:
        opt_problem['machine_rating'] = 15000000.0
        opt_problem['Torque']         = 20.64e6
        opt_problem['n_nom']          = 7.54
        opt_problem['machine_rating'] = 10000000.0
        opt_problem['Torque']         = 12.64e6
        
    #---------------------------------------------------
        
    # Specific costs
    opt_problem['C_Cu']         = 4.786         # Unit cost of Copper $/kg
    opt_problem['C_Fe']         = 0.556         # Unit cost of Iron $/kg
    opt_problem['C_Fes']        = 0.50139       # specific cost of Structural_mass $/kg
    
    #Material properties
    opt_problem['rho_Fe']       = 7700.0        # Steel density Kg/m3
    opt_problem['rho_Fes']      = 7850          # structural Steel density Kg/m3
    opt_problem['rho_Copper']   = 8900.0        # copper density Kg/m3
    opt_problem['rho_PM']       = 7450.0        # typical density Kg/m3 of neodymium magnets (added 2019 09 18) - for pmsg_[disc|arms]
            
    opt_problem['shaft_cm']     = np.zeros(3)
    opt_problem['shaft_length'] = 2.0
    
    #Run optimization
    opt_problem.model.approx_totals()
    opt_problem.run_driver()
    opt_problem.model.list_inputs(units=True) #values = False, hierarchical=False)
    opt_problem.model.list_outputs(units=True) #values = False, hierarchical=False)    

    # Export results
    if exportFlag:
        import pandas as pd

        if genType == 'scig':
            raw_data = {'Parameters': ['Rating',
                                       'Objective function',
                                       'Air gap diameter',
                                       'Stator length',
                                       'Lambda ratio',
                                       'Diameter ratio',
                                       'Pole pitch(tau_p)',
                                       'Number of Stator Slots',
                                       'Stator slot height(h_s)',
                                       'Stator slot width(b_s)',
                                       'Stator Slot aspect ratio',
                                       'Stator tooth width(b_t)',
                                       'Stator yoke height(h_ys)',
                                       'Rotor slots',
                                       'Rotor slot height(h_r)',
                                       'Rotor slot width(b_r)',
                                       'Rotor tooth width(b_tr)',
                                       'Rotor yoke height(h_yr)',
                                       'Rotor Slot_aspect_ratio',
                                       'Peak air gap flux density',
                                       'Peak air gap flux density fundamental',
                                       'Peak stator yoke flux density',
                                       'Peak rotor yoke flux density',
                                       'Peak Stator tooth flux density',
                                       'Peak Rotor tooth flux density',
                                       'Pole pairs',
                                       'Generator output frequency',
                                       'Generator output phase voltage',
                                       'Generator Output phase current',
                                       'Slip',
                                       'Stator Turns',
                                       'Conductor cross-section',
                                       'Stator Current density',
                                       'Specific current loading',
                                       'Stator resistance',
                                       'Excited magnetic inductance',
                                       'Magnetization current',
                                       'Conductor cross-section',
                                       'Rotor Current density',
                                       'Rotor resitance',
                                       'Generator Efficiency',
                                       'Overall drivetrain Efficiency',
                                       'Copper mass',
                                       'Iron Mass',
                                       'Structural mass',
                                       'Total Mass',
                                       'Total Material Cost'],
                        
                        'Values': [opt_problem['machine_rating']/1e6,
                                   Objective_function,
                                   2*opt_problem['r_s'],
                                   opt_problem['len_s'],
                                   opt_problem['K_rad'],
                                   opt_problem['D_ratio'],
                                   opt_problem['tau_p']*1000,
                                   opt_problem['S'],
                                   opt_problem['h_s']*1000,
                                   opt_problem['b_s']*1000,
                                   opt_problem['Slot_aspect_ratio1'],
                                   opt_problem['b_t']*1000,
                                   opt_problem['h_ys']*1000,
                                   opt_problem['Q_r'],
                                   opt_problem['h_r']*1000,
                                   opt_problem['b_r']*1000,
                                   opt_problem['b_tr']*1000,
                                   opt_problem['h_yr']*1000,
                                   opt_problem['Slot_aspect_ratio2'],
                                   opt_problem['B_g'],
                                   opt_problem['B_g1'],
                                   opt_problem['B_symax'],
                                   opt_problem['B_rymax'],
                                   opt_problem['B_tsmax'],
                                   opt_problem['B_trmax'],
                                   opt_problem['p'],
                                   opt_problem['f'],
                                   opt_problem['E_p'],
                                   opt_problem['I_s'],
                                   opt_problem['S_N'],
                                   opt_problem['N_s'],
                                   opt_problem['A_Cuscalc'],
                                   opt_problem['J_s'],
                                   opt_problem['A_1']/1000,
                                   opt_problem['R_s'],
                                   opt_problem['L_sm'],
                                   opt_problem['I_0'],
                                   opt_problem['A_bar']*1e6,
                                   opt_problem['J_r'],
                                   opt_problem['R_R'],
                                   opt_problem['gen_eff'],
                                   opt_problem['Overall_eff'],
                                   opt_problem['Copper']/1000,
                                   opt_problem['Iron']/1000,
                                   opt_problem['Structural_mass']/1000,
                                   opt_problem['Mass']/1000,
                                   opt_problem['Costs']/1000],
                        
                        'Limit': ['',
                                  '',
                                  '',
                                  '',
                                  '('+str(opt_problem['K_rad_LL'])+'-'+str(opt_problem['K_rad_UL'])+')',
                                  '('+str(opt_problem['D_ratio_LL'])+'-'+str(opt_problem['D_ratio_UL'])+')',
                                  
                                  '',
                                  '',
                                  '',
                                  '',
                                  '(4-10)',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '(4-10)',
                                  '(0.7-1.2)',
                                  '',
                                  '2',
                                  '2',
                                  '2',
                                  '2',
                                  '',
                                  '(10-60)',
                                  '(500-5000)',
                                  '',
                                  '(-30% to -0.2%)',
                                  '',
                                  '',
                                  '(3-6)',
                                  '<60',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  Eta_Target,
                                  '',
                                  '',
                                  '',
                                  '',
                                  ''],
                        
                        'Units':['MW',
                                 '',
                                 'm',
                                 'm',
                                 '-',
                                 '-',
                                 'mm',
                                 '-',
                                 'mm',
                                 'mm',
                                 '-',
                                 'mm',
                                 'mm',
                                 '-',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 '',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 '-',
                                 'Hz',
                                 'V',
                                 'A',
                                 '%',
                                 'turns',
                                 'mm^2',
                                 'A/mm^2',
                                 'kA/m',
                                 'ohms',
                                 'p.u',
                                 'A',
                                 'mm^2',
                                 'A/mm^2',
                                 'ohms',
                                 '%',
                                 '%',
                                 'Tons',
                                 'Tons',
                                 'Tons',
                                 'Tons',
                                 '$1000']}
            
        elif genType == 'dfig':
            raw_data = {'Parameters': ['Rating',
                                       'Objective function',
                                       'Air gap diameter',
                                       'Stator length',
                                       'K_rad',
                                       'Diameter ratio',
                                       'Pole pitch(tau_p)',
                                       ' Number of Stator Slots',
                                       'Stator slot height(h_s)',
                                       'Slots/pole/phase',
                                       'Stator slot width(b_s)',
                                       ' Stator slot aspect ratio',
                                       'Stator tooth width(b_t)',
                                       'Stator yoke height(h_ys)',
                                       'Rotor slots',
                                       'Rotor yoke height(h_yr)',
                                       'Rotor slot height(h_r)',
                                       'Rotor slot width(b_r)',
                                       ' Rotor Slot aspect ratio',
                                       'Rotor tooth width(b_t)',
                                       'Peak air gap flux density',
                                       'Peak air gap flux density fundamental',
                                       'Peak stator yoke flux density',
                                       'Peak rotor yoke flux density',
                                       'Peak Stator tooth flux density',
                                       'Peak rotor tooth flux density',
                                       'Pole pairs',
                                       'Generator output frequency',
                                       'Generator output phase voltage',
                                       'Generator Output phase current',
                                       'Optimal Slip',
                                       'Stator Turns',
                                       'Conductor cross-section',
                                       'Stator Current density',
                                       'Specific current loading',
                                       'Stator resistance',
                                       'Stator leakage inductance',
                                       'Excited magnetic inductance',
                                       'Rotor winding turns',
                                       'Conductor cross-section',
                                       'Magnetization current',
                                       'I_mag/Is',
                                       'Rotor Current density',
                                       'Rotor resitance',
                                       'Rotor leakage inductance',
                                       'Generator Efficiency',
                                       'Overall drivetrain Efficiency',
                                       'Copper mass',
                                       'Iron mass',
                                       'Structural Steel mass',
                                       'Total Mass',
                                       'Total Material Cost'],
                        
                        'Values': [opt_problem['machine_rating']/1e6,
                                   Objective_function,
                                   2*opt_problem['r_s'],
                                   opt_problem['len_s'],
                                   opt_problem['K_rad'],
                                   opt_problem['D_ratio'],
                                   opt_problem['tau_p']*1000,
                                   opt_problem['S'],
                                   opt_problem['h_s']*1000,
                                   opt_problem['q1'],
                                   opt_problem['b_s']*1000,
                                   opt_problem['Slot_aspect_ratio1'],
                                   opt_problem['b_t']*1000,
                                   opt_problem['h_ys']*1000,
                                   opt_problem['Q_r'],
                                   opt_problem['h_yr']*1000,
                                   opt_problem['h_r']*1000,
                                   opt_problem['b_r']*1000,
                                   opt_problem['Slot_aspect_ratio2'],
                                   opt_problem['b_tr']*1000,
                                   opt_problem['B_g'],
                                   opt_problem['B_g1'],
                                   opt_problem['B_symax'],
                                   opt_problem['B_rymax'],
                                   opt_problem['B_tsmax'],
                                   opt_problem['B_trmax'],
                                   opt_problem['p'],
                                   opt_problem['f'],
                                   opt_problem['E_p'],
                                   opt_problem['I_s'],
                                   opt_problem['S_Nmax'],
                                   opt_problem['N_s'],
                                   opt_problem['A_Cuscalc'],
                                   opt_problem['J_s'],
                                   opt_problem['A_1']/1000,
                                   opt_problem['R_s'],
                                   opt_problem['L_s'],
                                   opt_problem['L_sm'],
                                   opt_problem['N_r'],
                                   opt_problem['A_Curcalc'],
                                   opt_problem['I_0'],
                                   opt_problem['Current_ratio'],
                                   opt_problem['J_r'],
                                   opt_problem['R_R'],
                                   opt_problem['L_r'],
                                   opt_problem['gen_eff'],
                                   opt_problem['Overall_eff'],
                                   opt_problem['Copper']/1000,
                                   opt_problem['Iron']/1000,
                                   opt_problem['Structural_mass']/1000,
                                   opt_problem['Mass']/1000,
                                   opt_problem['Costs']/1000],
                        
                        'Limit': ['',
                                  '',
                                  '',
                                  '',
                                  '(0.2-1.5)',
                                  '(1.37-1.4)',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '(4-10)',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '(0.7-1.2)',
                                  '',
                                  '2.',
                                  '2.',
                                  '2.',
                                  '2.',
                                  '',
                                  '',
                                  '(500-5000)',
                                  '',
                                  '(-0.002-0.3)',
                                  '',
                                  '',
                                  '(3-6)',
                                  '<60',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '(0.1-0.3)',
                                  '(3-6)',
                                  '',
                                  '',
                                  '',
                                  Eta_Target,
                                  '',
                                  '',
                                  '',
                                  '',
                                  ''],
                        
                        'Units':['MW',
                                 '',
                                 'm',
                                 'm',
                                 '-',
                                 '-',
                                 'mm',
                                 '-',
                                 'mm',
                                 '',
                                 'mm',
                                 '',
                                 'mm',
                                 'mm',
                                 '-',
                                 'mm',
                                 'mm',
                                 'mm',
                                 '-',
                                 'mm',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 '-',
                                 'Hz',
                                 'V',
                                 'A',
                                 '',
                                 
                                 'turns',
                                 'mm^2',
                                 'A/mm^2',
                                 'kA/m',
                                 'ohms',
                                 '',
                                 '',
                                 'turns',
                                 'mm^2',
                                 'A',
                                 '',
                                 'A/mm^2',
                                 'ohms',
                                 'p.u',
                                 '%',
                                 '%',
                                 'Tons',
                                 'Tons',
                                 'Tons',
                                 'Tons',
                                 '$1000']}

        elif genType == 'eesg':
            raw_data = {'Parameters': ['Rating',
                                       'Stator Arms',
                                       'Stator Axial arm dimension',
                                       'Stator Circumferential arm dimension',
                                       'Stator arm Thickness',
                                       'Rotor Arms',
                                       'Rotor Axial arm dimension',
                                       'Rotor Circumferential arm dimension',
                                       'Rotor Arm thickness',
                                       'Rotor Radial deflection',
                                       'Rotor Axial deflection',
                                       'Rotor circum deflection',
                                       'Stator Radial deflection',
                                       'Stator Axial deflection',
                                       'Stator Circumferential deflection',
                                       'Air gap diameter',
                                       'Stator length',
                                       'l/D ratio',
                                       'Pole pitch',
                                       'Stator slot height',
                                       'Stator slot width',
                                       'Slot aspect ratio',
                                       'Stator tooth width',
                                       'Stator yoke height',
                                       'Rotor yoke height',
                                       'Rotor pole height',
                                       'Rotor pole width',
                                       'Average no-load flux density',
                                       'Peak air gap flux density',
                                       'Peak stator yoke flux density',
                                       'Peak rotor yoke flux density',
                                       'Stator tooth flux density',
                                       'Rotor pole core flux density',
                                       'Pole pairs',
                                       'Generator output frequency',
                                       'Generator output phase voltage(rms value)',
                                       'Generator Output phase current',
                                       'Stator resistance',
                                       'Synchronous inductance',
                                       'Stator slots',
                                       'Stator turns',
                                       'Stator conductor cross-section',
                                       'Stator Current density ',
                                       'Specific current loading',
                                       'Field turns',
                                       'Conductor cross-section',
                                       'Field Current',
                                       'D.C Field resistance',
                                       'MMF ratio at rated load(Rotor/Stator)',
                                       'Excitation Power (% of Rated Power)',
                                       'Number of brushes/polarity',
                                       'Field Current density',
                                       'Generator Efficiency',
                                       'Iron mass',
                                       'Copper mass',
                                       'Mass of Arms',
                                       'Total Mass',
                                       'Total Cost'],
                        
                        'Values': [opt_problem['machine_rating']/1e6,
                                   opt_problem['n_s'],
                                   opt_problem['d_s']*1000,
                                   opt_problem['b_st']*1000,
                                   opt_problem['t_ws']*1000,
                                   opt_problem['n_r'],
                                   opt_problem['d_r']*1000,
                                   opt_problem['b_r']*1000,
                                   opt_problem['t_wr']*1000,
                                   opt_problem['u_Ar']*1000,
                                   opt_problem['y_Ar']*1000,
                                   opt_problem['z_A_r']*1000,
                                   opt_problem['u_As']*1000,
                                   opt_problem['y_As']*1000,
                                   opt_problem['z_A_s']*1000,
                                   2*opt_problem['r_s'],
                                   opt_problem['len_s'],
                                   opt_problem['K_rad'],
                                   opt_problem['tau_p']*1000,
                                   opt_problem['h_s']*1000,
                                   opt_problem['b_s']*1000,
                                   opt_problem['Slot_aspect_ratio'],
                                   opt_problem['b_t']*1000,
                                   opt_problem['h_ys']*1000,
                                   opt_problem['h_yr']*1000,
                                   opt_problem['h_p']*1000,
                                   opt_problem['b_p']*1000,
                                   opt_problem['B_gfm'],
                                   opt_problem['B_g'],
                                   opt_problem['B_symax'],
                                   opt_problem['B_rymax'],
                                   opt_problem['B_tmax'],
                                   opt_problem['B_pc'],
                                   opt_problem['p'],
                                   opt_problem['f'],
                                   opt_problem['E_s'],
                                   opt_problem['I_s'],
                                   opt_problem['R_s'],
                                   opt_problem['L_m'],
                                   opt_problem['S'],
                                   opt_problem['N_s'],
                                   opt_problem['A_Cuscalc'],
                                   opt_problem['J_s'],
                                   opt_problem['A_1']/1000,
                                   opt_problem['N_f'],
                                   opt_problem['A_Curcalc'],
                                   opt_problem['I_f'],
                                   opt_problem['R_r'],
                                   opt_problem['Load_mmf_ratio'],
                                   opt_problem['Power_ratio'],
                                   opt_problem['n_brushes'],
                                   opt_problem['J_f'],
                                   opt_problem['gen_eff'],
                                   opt_problem['Iron']/1000,
                                   opt_problem['Copper']/1000,
                                   opt_problem['Structural_mass']/1000,
                                   opt_problem['Mass']/1000,
                                   opt_problem['Costs']/1000],
                        
                        'Limit': ['',
                                  '',
                                  '',
                                  opt_problem['b_all_s']*1000,
                                  '',
                                  '',
                                  '',
                                  opt_problem['b_all_r']*1000,
                                  '',
                                  opt_problem['u_all_r']*1000,
                                  opt_problem['y_all']*1000,
                                  opt_problem['z_all_r']*1000,
                                  opt_problem['u_all_s']*1000,
                                  opt_problem['y_all']*1000,
                                  opt_problem['z_all_s']*1000,
                                  
                                  '',
                                  '',
                                  '(0.2-0.27)',
                                  '',
                                  '',
                                  '',
                                  '(4-10)',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '(0.62-1.05)',
                                  '1.2',
                                  '2',
                                  '2',
                                  '2',
                                  '2',
                                  '',
                                  '(10-60)',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '(3-6)',
                                  '<60',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '<2%',
                                  '',
                                  '(3-6)',
                                  Eta_Target,
                                  '',
                                  '',
                                  '',
                                  '',
                                  ''],
                        
                        'Units':['MW',
                                 'unit',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'unit',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'm',
                                 'm',
                                 '',
                                 '',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 '-',
                                 'Hz',
                                 'V',
                                 'A',
                                 'om/phase',
                                 
                                 'p.u',
                                 'slots',
                                 'turns',
                                 'mm^2',
                                 'A/mm^2',
                                 'kA/m',
                                 'turns',
                                 'mm^2',
                                 'A',
                                 'ohm',
                                 '%',
                                 '%',
                                 'brushes',
                                 'A/mm^2',
                                 'turns',
                                 '%',
                                 'tons',
                                 'tons',
                                 'tons',
                                 '1000$']} 

        elif genType == 'pmsg_arms':
                raw_data = {'Parameters': ['Rating',
                                           'Stator Arms',
                                           'Stator Axial arm dimension',
                                           'Stator Circumferential arm dimension',
                                           'Stator arm Thickness',
                                           'Rotor arms',
                                           'Rotor Axial arm dimension',
                                           'Rotor Circumferential arm dimension',
                                           'Rotor arm Thickness',
                                           'Stator Radial deflection',
                                           'Stator Axial deflection',
                                           'Stator circum deflection',
                                           'Rotor Radial deflection',
                                           'Rotor Axial deflection',
                                           'Rotor circum deflection',
                                           'Air gap diameter',
                                           'Overall Outer diameter',
                                           'Stator length',
                                           'l/d ratio',
                                           'Slot_aspect_ratio',
                                           'Pole pitch',
                                           'Stator slot height',
                                           'Stator slotwidth',
                                           'Stator tooth width',
                                           'Stator yoke height',
                                           'Rotor yoke height',
                                           'Magnet height',
                                           'Magnet width',
                                           'Peak air gap flux density fundamental',
                                           'Peak stator yoke flux density',
                                           'Peak rotor yoke flux density',
                                           'Flux density above magnet',
                                           'Maximum Stator flux density',
                                           'Maximum tooth flux density',
                                           'Pole pairs',
                                           'Generator output frequency',
                                           'Generator output phase voltage',
                                           'Generator Output phase current',
                                           'Stator resistance',
                                           'Synchronous inductance',
                                           'Stator slots',
                                           'Stator turns',
                                           'Conductor cross-section',
                                           'Stator Current density ',
                                           'Specific current loading',
                                           'Generator Efficiency ',
                                           'Iron mass',
                                           'Magnet mass',
                                           'Copper mass',
                                           'Mass of Arms',
                                           'Total Mass',
                                           'Total Material Cost'],
                            
                        'Values': [opt_problem['machine_rating']/1000000,
                                   opt_problem['n_s'],
                                   opt_problem['d_s']*1000,
                                   opt_problem['b_st']*1000,
                                   opt_problem['t_ws']*1000,
                                   opt_problem['n_r'],
                                   opt_problem['d_r']*1000,
                                   opt_problem['b_r']*1000,
                                   opt_problem['t_wr']*1000,
                                   opt_problem['u_As']*1000,
                                   opt_problem['y_As']*1000,
                                   opt_problem['z_A_s']*1000,
                                   opt_problem['u_Ar']*1000,
                                   opt_problem['y_Ar']*1000,
                                   opt_problem['z_A_r']*1000,
                                   2*opt_problem['r_s'],
                                   opt_problem['R_out']*2,
                                   opt_problem['len_s'],
                                   opt_problem['K_rad'],
                                   opt_problem['Slot_aspect_ratio'],
                                   opt_problem['tau_p']*1000,
                                   opt_problem['h_s']*1000,
                                   opt_problem['b_s']*1000,
                                   opt_problem['b_t']*1000,
                                   opt_problem['h_ys']*1000,
                                   opt_problem['h_yr']*1000,
                                   opt_problem['h_m']*1000,
                                   opt_problem['b_m']*1000,
                                   opt_problem['B_g'],
                                   opt_problem['B_symax'],
                                   opt_problem['B_rymax'],
                                   opt_problem['B_pm1'],
                                   opt_problem['B_smax'],
                                   opt_problem['B_tmax'],
                                   opt_problem['p'],
                                   opt_problem['f'],
                                   opt_problem['E_p'],
                                   opt_problem['I_s'],
                                   opt_problem['R_s'],
                                   opt_problem['L_s'],
                                   opt_problem['S'],
                                   opt_problem['N_s'],
                                   opt_problem['A_Cuscalc'],
                                   opt_problem['J_s'],
                                   opt_problem['A_1']/1000,
                                   opt_problem['gen_eff'],
                                   opt_problem['Iron']/1000,
                                   opt_problem['mass_PM']/1000,
                                   opt_problem['Copper']/1000,
                                   opt_problem['Structural_mass']/1000,
                                   opt_problem['Mass']/1000,
                                   opt_problem['Costs']/1000],
                            
                        'Limit': ['',
                                  '',
                                  '',
                                  opt_problem['b_all_s']*1000,
                                  '',
                                  '',
                                  '',
                                  opt_problem['b_all_r']*1000,
                                  '',
                                  opt_problem['u_all_s']*1000,
                                  opt_problem['y_all']*1000,
                                  opt_problem['z_all_s']*1000,
                                  opt_problem['u_all_r']*1000,
                                  opt_problem['y_all']*1000,
                                  opt_problem['z_all_r']*1000,
                                  '',
                                  '',
                                  '',
                                  '(0.2-0.27)',
                                  '(4-10)',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '<2',
                                  '<2',
                                  '<2',
                                  opt_problem['B_g'],
                                  '',
                                  '',
                                  '',
                                  '',
                                  '>500',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '5',
                                  '3-6',
                                  '60',
                                  '>93%',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  ''],
                            
                        'Units':['MW',
                                 'unit',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 '',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'm',
                                 'm',
                                 'm',
                                 '',
                                 '',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 '-',
                                 'Hz',
                                 'V',
                                 'A',
                                 'ohm/phase',
                                 'p.u',
                                 'slots',
                                 'turns',
                                 'mm^2',
                                 'A/mm^2',
                                 'kA/m',
                                 '%',
                                 'tons',
                                 'tons',
                                 'tons',
                                 'tons',
                                 'tons',
                                 'k$']}

        elif genType == 'pmsg_disc':
            raw_data = {'Parameters': ['Rating',
                                       'Stator Arms',
                                       'Stator Axial arm dimension',
                                       'Stator Circumferential arm dimension',
                                       'Stator arm Thickness',
                                       'Rotor disc Thickness',
                                       'Stator Radial deflection',
                                       'Stator Axial deflection',
                                       'Stator circum deflection',
                                       'Rotor Radial deflection',
                                       'Rotor Axial deflection',
                                       'Air gap diameter',
                                       'Overall Outer diameter',
                                       'Stator length',
                                       'l/d ratio',
                                       'Slot_aspect_ratio',
                                       'Pole pitch',
                                       'Stator slot height',
                                       'Stator slotwidth',
                                       'Stator tooth width',
                                       'Stator yoke height',
                                       'Rotor yoke height',
                                       'Magnet height',
                                       'Magnet width',
                                       'Peak air gap flux density fundamental',
                                       'Peak stator yoke flux density',
                                       'Peak rotor yoke flux density',
                                       'Flux density above magnet',
                                       'Maximum Stator flux density',
                                       'Maximum tooth flux density',
                                       'Pole pairs',
                                       'Generator output frequency',
                                       'Generator output phase voltage',
                                       'Generator Output phase current',
                                       'Stator resistance',
                                       'Synchronous inductance',
                                       'Stator slots',
                                       'Stator turns',
                                       'Conductor cross-section',
                                       'Stator Current density ',
                                       'Specific current loading',
                                       'Generator Efficiency ',
                                       'Iron mass',
                                       'Magnet mass',
                                       'Copper mass',
                                       'Mass of Arms and disc',
                                       'Total Mass',
                                       'Total Material Cost'],
                        
                        'Values': [opt_problem['machine_rating']/1000000,
                                   opt_problem['n_s'],
                                   opt_problem['d_s']*1000,
                                   opt_problem['b_st']*1000,
                                   opt_problem['t_ws']*1000,
                                   opt_problem['t_d']*1000,
                                   opt_problem['u_As']*1000,
                                   opt_problem['y_As']*1000,
                                   opt_problem['z_A_s']*1000,
                                   opt_problem['u_Ar']*1000,
                                   opt_problem['y_Ar']*1000,
                                   2*opt_problem['r_s'],
                                   opt_problem['R_out']*2,
                                   opt_problem['len_s'],
                                   opt_problem['K_rad'],
                                   opt_problem['Slot_aspect_ratio'],
                                   opt_problem['tau_p']*1000,
                                   opt_problem['h_s']*1000,
                                   opt_problem['b_s']*1000,
                                   opt_problem['b_t']*1000,
                                   opt_problem['h_ys']*1000,
                                   opt_problem['h_yr']*1000,
                                   opt_problem['h_m']*1000,
                                   opt_problem['b_m']*1000,
                                   opt_problem['B_g'],
                                   opt_problem['B_symax'],
                                   opt_problem['B_rymax'],
                                   opt_problem['B_pm1'],
                                   opt_problem['B_smax'],
                                   opt_problem['B_tmax'],
                                   opt_problem['p'],
                                   opt_problem['f'],
                                   opt_problem['E_p'],
                                   opt_problem['I_s'],
                                   opt_problem['R_s'],
                                   opt_problem['L_s'],
                                   opt_problem['S'],
                                   opt_problem['N_s'],
                                   opt_problem['A_Cuscalc'],
                                   opt_problem['J_s'],
                                   opt_problem['A_1']/1000,
                                   opt_problem['gen_eff'],
                                   opt_problem['Iron']/1000,
                                   opt_problem['mass_PM']/1000,
                                   opt_problem['Copper']/1000,
                                   opt_problem['Structural_mass']/1000,
                                   opt_problem['Mass']/1000,
                                   opt_problem['Costs']/1000],
                        
                        'Limit': ['',
                                  '',
                                  '',
                                  opt_problem['b_all_s']*1000,
                                  '',
                                  '',
                                  opt_problem['u_all_s']*1000,
                                  opt_problem['y_all']*1000,
                                  opt_problem['z_all_s']*1000,
                                  opt_problem['u_all_r']*1000,
                                  opt_problem['y_all']*1000,
                                  '',
                                  '',
                                  '',
                                  '(0.2-0.27)',
                                  '(4-10)',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '<2',
                                  '<2',
                                  '<2',
                                  '<2',
                                  '<2',
                                  opt_problem['B_g'],
                                  '<2',
                                  '',
                                  '>500',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '5',
                                  
                                  '3-6',
                                  '60',
                                  '>93%',
                                  '',
                                  '',
                                  '',
                                  '',
                                  '',
                                  ''],
                        
                        'Units':['MW',
                                 'unit',
                                 'mm',
                                 'mm',
                                 'mm',
                                 '',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'm',
                                 'm',
                                 'm',
                                 '',
                                 '',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'mm',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 'T',
                                 '-',
                                 'Hz',
                                 'V',
                                 'A',
                                 'ohm/phase',
                                 'p.u',
                                 'slots',
                                 'turns',
                                 'mm^2',
                                 'A/mm^2',
                                 'kA/m',
                                 '%',
                                 'tons',
                                 'tons',
                                 'tons',
                                 'tons',
                                 'tons',
                                 'k$']}
                
            
        df = pd.DataFrame(raw_data, columns=['Parameters','Values','Limit','Units'])
        print(df)
        df.to_excel(genType.upper() + '_' + str(float(opt_problem['machine_rating']/1e6)) + '_MW.xlsx')
        df.to_csv(genType.upper() + '_' + str(float(opt_problem['machine_rating']/1e6)) + '_MW.csv')
            
if __name__=='__main__':
    
    # Run example optimizations for all generator types
    #for m in ['eesg',]:
    #for m in ['scig','dfig','eesg','pmsg_arms','pmsg_disc']:
    for m in ['pmsg_disc']:
        optimization_example(m, exportFlag=True)
